/*
** curgtk.c
*/

#include <gtk/gtk.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#define TILE_WIDTH	8
#define TILE_HEIGHT	12

#define SCREEN_WIDTH (80 * TILE_WIDTH)
#define SCREEN_HEIGHT (40 * TILE_HEIGHT)

#define RESET_SCREEN_RECT() \
  screen_rect.x      = SCREEN_WIDTH; \
  screen_rect.y      = SCREEN_HEIGHT; \
  screen_rect.width  = 0; \
  screen_rect.height = 0;

#define SET_SCREEN_RECT() \
  screen_rect.x      = 0; \
  screen_rect.y      = 0; \
  screen_rect.width  = SCREEN_WIDTH; \
  screen_rect.height = SCREEN_HEIGHT;

void UserProgramAbout( void );

typedef unsigned short chtype;

extern int pLINES;
extern int pCOLS;

long sleepTime;

gint theKey;
gint userHitKey;

GdkRectangle screen_rect = {0,0,0,0};

GdkPixmap * win_pixmap;
GdkPixmap * charset_pixmap;

GtkWidget * curses_window;
GtkWidget * screen_da;

short mac_getchar( void );
void bigpixel_putchar( chtype c, short x, short y );
void smallpixel_putchar( chtype c, short x, short y );
void mac_update( short line, short beg, short end );
void mac_scroll( void );
void mac_beep( void );
void mac_clear( void );

/*
static GdkRectangle union_rect (GdkRectangle r1, GdkRectangle r2)
{
  GdkRectangle result;

  if (r1.x < r2.x) result.x = r1.x;
  else             result.x = r2.x;

  if (r1.y < r2.y) result.y = r1.y;
  else             result.y = r2.y;

  if ((r1.x + r1.width) > (r2.x + r2.width)) result.width = r1.x + r1.width - result.x;
  else                                       result.width = r2.x + r2.width - result.x;

  if ((r1.y + r1.height) > (r2.y + r2.height)) result.height = r1.y + r1.height - result.y;
  else                                         result.height = r2.y + r2.height - result.y;

  return result;
}
*/

static gint expose_event (GtkWidget * widget, GdkEventExpose * event)
{
  GdkGC * gc;
  GdkWindow * win;
  GdkRectangle prect;

  gc = widget->style->fg_gc[GTK_WIDGET_STATE(widget)];
  win = widget->window;

  if (0 == event->area.width || 0 == event->area.height)
    return FALSE;

  if (event->area.x >= SCREEN_WIDTH || event->area.y >= SCREEN_HEIGHT)
    return FALSE;

  prect = event->area;
  gdk_draw_pixmap(win, gc, win_pixmap,
                  prect.x, prect.y,
                  prect.x, prect.y,
                  prect.width, prect.height);

  return FALSE;
}

static gint configure_event (GtkWidget * widget, GdkEventConfigure * event)
{
  gint width;
  gint height;
  GtkStyle * style;
  GdkWindow * win;

  if (win_pixmap) gdk_pixmap_unref(win_pixmap);

  win = widget->window;
  width = widget->allocation.width;
  height = widget->allocation.height;

  win_pixmap = gdk_pixmap_new(win, width, height, -1);

  style = gtk_widget_get_style(curses_window);
  gdk_draw_rectangle(win_pixmap, style->black_gc,
                     1, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);

  if (!charset_pixmap)
    {
      GdkBitmap * mask;

      charset_pixmap =
        gdk_pixmap_create_from_xpm(curses_window->window, &mask,
                                   &(style->bg[GTK_STATE_NORMAL]), "./omegalib/omegatiles.xpm");

      if (0 == charset_pixmap)
        {
          printf("Unable to load the file 'omegatiles.xpm'\n");
          exit(1);
        }
    }

  return TRUE;
}

void close_mac_curses ( void )
{
  /* gtk_main_quit(); */
}

static void quit (GtkWidget * widget, gpointer data)
{
  close_mac_curses();
}

static gint key_press_event (GtkWidget * widget, GdkEventKey * event)
{
  event->keyval &= 0xFF;

  if (event->keyval < 0x80)
    {
      if (event->state & 4)
        event->keyval &= 0x1F;

      theKey = event->keyval;
      userHitKey = TRUE;
    }

  event->keyval = 0;

  return TRUE;
}

gint curgtk_timed (gpointer data)
{
  gtk_widget_draw((GtkWidget *)screen_da, &screen_rect);
  RESET_SCREEN_RECT();

  return TRUE;
}

int init_mac_curses(void)
{
  GtkWidget * vbox;

  int argc;
  char ** argv;

  argc = 1;
  argv = (char **)malloc(2 * sizeof(char *));
  argv[0] = (char *)malloc(8);
  argv[1] = 0;
  strcpy(argv[0], "program");

  gtk_init(&argc, &argv);

  sleepTime = 10;
  userHitKey = FALSE;
  pCOLS = 80;
  pLINES = 40;

  /* create the curses window */

  curses_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_events(curses_window, GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);

  // gtk_widget_set_name(curses_window, "Omega");

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(curses_window), vbox);
  gtk_widget_show(vbox);

  gtk_signal_connect(GTK_OBJECT(curses_window), "destroy", GTK_SIGNAL_FUNC(quit), NULL);

  /* create the drawing area */

  screen_da = gtk_drawing_area_new();
  gtk_widget_set_events(screen_da, GDK_EXPOSURE_MASK);
  gtk_drawing_area_size(GTK_DRAWING_AREA(screen_da), SCREEN_WIDTH, SCREEN_HEIGHT);
  gtk_box_pack_start(GTK_BOX(vbox), screen_da, TRUE, TRUE, 0);
  gtk_widget_show(screen_da);

  /* signals used to handle backing pixmap */

  gtk_signal_connect(GTK_OBJECT(screen_da), "expose_event",
                     (GtkSignalFunc)expose_event, NULL);
  gtk_signal_connect(GTK_OBJECT(screen_da), "configure_event",
                     (GtkSignalFunc)configure_event, NULL);

  /* keyboard signals */

  gtk_signal_connect(GTK_OBJECT(curses_window), "key_press_event",
                     (GtkSignalFunc)key_press_event, NULL);

  gtk_widget_show(curses_window);

  SET_SCREEN_RECT();
  gtk_widget_draw((GtkWidget *)screen_da, &screen_rect);
  RESET_SCREEN_RECT();

  gtk_main_iteration();

  // gtk_timeout_add(1000 / 30, curgtk_timed, 0);

  return TRUE;
}

/*
short mac_getchar( void )
{
	time_t otime, ntime;
	
	ntime = otime = clock();
	while (( ntime - otime ) < sleepTime )
	{
		gotAnEvent = WaitNextEvent( everyEvent, &myNewEvent, sleepTime, cursorRegion );
		if ( !gotAnEvent )
			ntime = clock();
		else
		{
			HandleEvent( &myNewEvent );
			if ( userHasQuit )
			{
				userHasQuit = FALSE;
				raise( SIGINT );
			}
			else if ( userHitKey )
			{
				userHitKey = FALSE;
				return( (int)theKey );
			}
			else
			{
				gotAnEvent = FALSE;
				ntime = clock();
			}
		}
	}
	return -1;
}
*/

short mac_getchar( void )
{
  while (gtk_events_pending())
    gtk_main_iteration();

  if (userHitKey)
    {
      userHitKey = FALSE;
      return theKey;
    }

  return -1;
}

void mac_putchar( chtype c, short x, short y )
{
  gint col, line;
  GdkRectangle my_rect;

  col = 1 + ((TILE_WIDTH + 1) * (c % 32));
  line = 1 + ((TILE_HEIGHT + 1) * (c / 32));

  my_rect.x      = TILE_WIDTH * x;
  my_rect.y      = TILE_HEIGHT * y;
  my_rect.width  = TILE_WIDTH;
  my_rect.height = TILE_HEIGHT;

  gdk_draw_pixmap(win_pixmap, ((GtkWidget *)(screen_da))->style->black_gc, charset_pixmap,
                  col, line, my_rect.x, my_rect.y, my_rect.width, my_rect.height);


  /*
  gtk_widget_draw((GtkWidget *)screen_da, &my_rect);

  screen_rect = union_rect(my_rect, screen_rect);

  while (gtk_events_pending())
    gtk_main_iteration();
  */
}

void mac_update( short line, short beg, short end )
{
  GdkRectangle my_rect;

  my_rect.x = TILE_WIDTH * beg;
  my_rect.y = TILE_HEIGHT * line;
  my_rect.width = (end - beg + 1) * TILE_WIDTH;
  my_rect.height = TILE_HEIGHT;

  gtk_widget_draw((GtkWidget *)screen_da, &my_rect);

  while (gtk_events_pending())
    gtk_main_iteration();
}

void mac_scroll( void )
{
}

void mac_beep( void )
{
}

void mac_clear( void )
{
  GtkStyle * style;

  style = gtk_widget_get_style(curses_window);
  gdk_draw_rectangle(win_pixmap, style->black_gc,
                     1, 0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);

  SET_SCREEN_RECT();
}
