/*
** xcurses.h - Mac curses
*/

#define FALSE 0
#define TRUE 1

#define ACS_ULCORNER 0x80
#define ACS_LLCORNER 0x81
#define ACS_URCORNER 0x82
#define ACS_LRCORNER 0x83
#define ACS_VLINE    0x89
#define ACS_HLINE    0x88

typedef unsigned short chtype;
typedef char bool;

typedef struct window
{
	short curX, curY, lines, cols;
	short left, top, right, bottom;
	short type;
	unsigned short attrword;
	bool f_leave, f_scroll, f_clear, f_standout, f_nodelay;
	chtype **winBuf;
	short *begLine, *endLine;
	struct window *next;
} WINDOW;

/* These are prototypes from curmac.c */
int init_mac_curses( void );
void close_mac_curses( void );
short mac_getchar( void );
void mac_putchar( chtype c, short x, short y );
void mac_update( short line, short beg, short end );
void mac_scroll( void );
void mac_beep( void );
void mac_clear( void );

/* These are prototypes from curses.c */
void scrollWindow( WINDOW *win, bool update );
void consolePutchar( WINDOW *win, chtype ch );
int consoleGetchar( WINDOW *win );
WINDOW *allocWindow( int y, int x, int lines, int cols );
void freeWindow( WINDOW *win );

int initscr( void );
int endwin( void );
WINDOW *newwin( int lines, int cols, int uly, int ulx );
WINDOW *subwin( WINDOW *orig, int lines, int cols, int y, int x );
int mvwin( WINDOW *win, int y, int x );
int delwin( WINDOW *win );
int touchwin( WINDOW *win );
int touchline( WINDOW *win, int start, int count );
int overlay( WINDOW *srcwin, WINDOW *dstwin );
int overwrite( WINDOW *srcwin, WINDOW *dstwin );
int copywin( WINDOW *src, WINDOW *dst, int sminrow, int smincol, int dminrow,
				 int dmincol, int dmaxrow, int dmaxcol, int overlay );
int box( WINDOW *win, chtype vert, chtype horiz );
int scroll( WINDOW *win );
int scrollok( WINDOW *win, bool flag );
int leaveok( WINDOW *win, bool flag );
int clearok( WINDOW *win, bool flag );
int idlok( WINDOW *win, bool flag );
int nodelay( WINDOW *win, bool flag );

int cbreak( void );
int nocbreak( void );
int raw( void );
int noraw( void );
int echo( void );
int noecho( void );
int nl( void );
int nonl( void );
int baudrate( void );
char erasechar( void );
char killchar( void );
char *longname( void );
int wattrset( WINDOW *win, int attrs );
int beep( void );
int flash( void );
int keypad( WINDOW *win, bool flag );

int wstandout( WINDOW *win );
int wstandend( WINDOW *win );
int waddch( WINDOW *win, chtype c );
int waddstr( WINDOW *win, char *s );
int wclear( WINDOW *win );
int wclrtobot( WINDOW *win );
int wclrtoeol( WINDOW *win );
int werase( WINDOW *win );
int wdelch( WINDOW *win );
int wdeleteln( WINDOW *win );
chtype wgetch( WINDOW *win );
char *wgetstr( WINDOW *win, char *s );
int winsertln( WINDOW *win );
int winsch( WINDOW *win, chtype c );
int wmove( WINDOW *win, int y, int x );
int wnoutrefresh( WINDOW *win );
int doupdate( void );
int wrefresh( WINDOW *win );

int printw( const char *fstr, ... );
int wprintw( WINDOW *win, const char *fstr, ... );
int vwprintw( WINDOW *win, const char *fstr, va_list arg );
int mvprintw( int y, int x, const char *fstr, ... );
int mvwprintw( WINDOW *win, int y, int x, const char *fstr, ... );
