/* omega copyright (C) by Laurence Raphael Brothers, 1987,1988,1989 */
/* country.c */
/* load_country(), and all load_ functions for places which are */
/* accessible from the country and don't have their own files */

#include "glob.h"
#include "ochars.h"

void set_correct_hedge( int i, int j, plv Level );
void set_correct_pool( int i, int j, plv Level );
void set_correct_city_wall( int i, int j, plv Level );

/* loads the countryside level from the data file */
void load_country( void )
{
	int i, j;
	short swap, site;
	FILE *fd;
  
	strcpy( Str3, Omegalib );
	strcat( Str3, "country.dat" );
	fd = checkfopen( Str3, "rb" );

	for( j = 0; j < LENGTH; j++ )
	{
		for( i = 0; i < WIDTH; i++ )
		{
			fread( &swap, sizeof( short ), 1, fd );
            site = ((swap & 0xFF00) >> 8) | ((swap & 0x00FF) << 8);
			Country[ i ][ j ].aux = 0;
			Country[ i ][ j ].status = 0;
			Country[ i ][ j ].base_terrain_type = site;
			Country[ i ][ j ].current_terrain_type = site;
			switch ( site )
			{
				case PASS_UL_PLAIN:
					Country[ i ][ j ].current_terrain_type = MTN_TP_LP;
					break;
					
				case PASS_ULTOLR:
					Country[ i ][ j ].current_terrain_type = MTN_WHOLE;
					break;
					
				case PASS_LR_FOREST:
					Country[ i ][ j ].current_terrain_type = MTN_BF;
					break;
					
				case PASS_L_PLAIN:
					Country[ i ][ j ].current_terrain_type = MTN_LP;
					break;
					
				case PASS_LTOR:
					Country[ i ][ j ].current_terrain_type = MTN_URF_LRP;
					break;
					
				case PASS_R_FOREST:
					Country[ i ][ j ].current_terrain_type = MTN_TF_RF_BP;
					break;
					
				case PASS_ULTOLL:
					Country[ i ][ j ].current_terrain_type = MTN_WHOLE;
					break;
					
				case PASS_LLTOUR:
					Country[ i ][ j ].current_terrain_type = MTN_WHOLE;
					break;
					
				case PASS_PENIN:
					Country[ i ][ j ].current_terrain_type = MTN_ULR_LRR;
					break;
					
				case MTN_ULPS_LRP:
					Country[ i ][ j ].current_terrain_type = MTN_LRP;
					break;
					
				case MTN_ULPS_LLPS_BP:
					Country[ i ][ j ].current_terrain_type = MTN_LLRD_BP;
					break;
					
				case MTN_LRPS:
					Country[ i ][ j ].current_terrain_type = MTN_WHOLE;
					break;
					
				case MTN_LLPS:
					Country[ i ][ j ].current_terrain_type = MTN_WHOLE;
					break;
					
				case MTN_URPS:
					Country[ i ][ j ].current_terrain_type = MTN_WHOLE;
					break;
					
				case MTN_URPS_LRPS:
					Country[ i ][ j ].current_terrain_type = MTN_WHOLE;
					break;
					
				case MTN_URPS_ULP:
					Country[ i ][ j ].current_terrain_type = MTN_ULP;
					break;
					
				case MTN_URPS_BF:
					Country[ i ][ j ].current_terrain_type = MTN_BF;
					break;
					
				case MTN_LLPS_TP:
					Country[ i ][ j ].current_terrain_type = MTN_TP;
					break;
					
				case C_CASTLE:
					Country[ i ][ j ].current_terrain_type = MTN_WHOLE;
					break;
					
				case C_STARPEAK:
					Country[ i ][ j ].current_terrain_type = MTN_PTIP;
					break;
					
				case C_CAVES:
					Country[ i ][ j ].current_terrain_type = MTN_ULP_LLP;
					break;
					
				case C_VOLCANO:
					Country[ i ][ j ].current_terrain_type = MTN_ULS_LLS_URS_LRS;
					break;
					
				case C_DRAGONLAIR:
					Country[ i ][ j ].current_terrain_type = MTN_LD;
					break;
					
				case C_MAGICISLE:
					Country[ i ][ j ].current_terrain_type = SWAMP_ISLE;
					break;
				
				case C_THAUMARIS:
					Country[ i ][ j ].current_terrain_type = C_THAUM_HP;
					Country[ i ][ j ].aux = 4;
					break;
					
				case C_WOODMERE:
					Country[ i ][ j ].aux = 2;
					break;
					
				case C_SKORCH:
					Country[ i ][ j ].aux = 5;
					break;
					
				case C_STARVIEW:
					Country[ i ][ j ].aux = 1;
					break;
					
				case C_STORMWATCH:
					Country[ i ][ j ].aux = 3;
					break;
					
				case C_WHORFEN:
					Country[ i ][ j ].aux = 6;
					break;
					
				case C_DRUID:
					Country[ i ][ j ].aux = DRUID;
					break;
					
				case C_ATHENA:
					Country[ i ][ j ].aux = ATHENA;
					break;
					
				case C_ODIN:
					Country[ i ][ j ].aux = ODIN;
					break;
					
				case C_SET:
					Country[ i ][ j ].aux = SET;
					break;
					
				case C_HECATE:
					Country[ i ][ j ].aux = HECATE;
					break;
					
				case C_DESTINY:
					Country[ i ][ j ].aux = DESTINY;
					break;
			}
		}
	}
	fclose( fd );
}

/* loads the dragon's lair into Level*/
void load_dlair( int empty, int populate )
{
	int i, j;
	char site;
	unsigned short saveloc;
	FILE *fd;

	if ( empty )
	{
		mprint( "The Lair is now devoid of inhabitants and treasure." );
		morewait();
	}

	if ( !populate )
		empty = TRUE;
	TempLevel = Level;
	if ( ok_to_free( TempLevel ))
	{
		free_level( TempLevel );
		TempLevel = NULL;
	}
	Level = ( (plv)checkmalloc( sizeof( levtype )));
	clear_level( Level );
	Level->environment = E_DLAIR;
	strcpy( Str3, Omegalib );
	strcat( Str3, "dlair.dat" );
	fd = checkfopen( Str3, "rb" );
	site = cryptkey( "dlair.dat" );
	
	for( j = 0; j < LENGTH; j++ )
	{
		for( i = 0; i < WIDTH; i++ )
		{
			Level->site[ i ][ j ].lstatus = 0;
			if ( i < 48 )
				Level->site[ i ][ j ].roomnumber = RS_CAVERN;
			else
				Level->site[ i ][ j ].roomnumber = RS_DRAGONLORD;
			Level->site[ i ][ j ].p_locf = L_NO_OP;
			site = getc( fd ) ^ site;
			switch( site )
			{
				case 'D':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
					{
						make_site_monster( i, j, ML10 + 3 ); /* dlord */
						Level->site[ i ][ j ].creature->specialf = M_SP_LAIR;
					}
					break;
					
				case 'd':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
					{
						make_site_monster( i, j, ML8 + 3 ); /* elite dragons */
						Level->site[ i ][ j ].creature->specialf = M_SP_LAIR;
						Level->site[ i ][ j ].creature->hit *= 2;
						Level->site[ i ][ j ].creature->dmg *= 2;
					}
					break;
					
				case 'W':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
						make_site_monster( i, j, ML9 + 2 );
					break;
					
				case 'M':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
						make_site_monster( i, j, -1 );
					break;
					
				case 'S':
					Level->site[ i ][ j ].locchar = GROUND;
					Level->site[ i ][ j ].showchar = WALL_WHOLE;
					if ( !empty )
						lset( i, j, SECRET );
					Level->site[ i ][ j ].roomnumber = RS_SECRETPASSAGE;
					break;
					
				case '$':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
						make_site_treasure( i, j, 10 );
					break;
					
				case 's':
					Level->site[ i ][ j ].locchar = GROUND;
					Level->site[ i ][ j ].p_locf = L_TRAP_SIREN;
					break;
					
				case '7':
					if ( !empty )
						Level->site[ i ][ j ].locchar = V_PORTCULLIS;
					else
						Level->site[ i ][ j ].locchar = V_GROOVE;
					Level->site[ i ][ j ].p_locf = L_PORTCULLIS;
					break;
					
				case 'R':
					Level->site[ i ][ j ].locchar = GROUND;
					Level->site[ i ][ j ].p_locf = L_RAISE_PORTCULLIS;
					break;
					
				case 'p':
					Level->site[ i ][ j ].locchar = V_GROOVE;
					Level->site[ i ][ j ].p_locf = L_PORTCULLIS;
					break;
					
				case 'T':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
						Level->site[ i ][ j ].p_locf = L_PORTCULLIS_TRAP;
					break;
					
				case 'X':
					Level->site[ i ][ j ].locchar = GROUND;
					Level->site[ i ][ j ].p_locf = L_TACTICAL_EXIT;
					break;
					
				case '#':
					Level->site[ i ][ j ].locchar = WALL_WHOLE;
					Level->site[ i ][ j ].aux = 150; 
					break;
					
				case '.':
					Level->site[ i ][ j ].locchar = GROUND;
					break;
			}
		}
		site = getc( fd ) ^ site;
	}
	
	for( j = 0; j < LENGTH; j++ )
	{
		for( i = 0; i < WIDTH; i++ )
		{
			switch( Level->site[ i ][ j ].locchar )
			{
				case WALL_WHOLE:
					set_correct_city_wall( i, j, Level );
					break;
					
				case POOL_WHOLE:
					set_correct_pool( i, j, Level );
					break;
					
				case HEDGE_WHOLE:
					set_correct_hedge( i, j, Level );
					break;
			}

			if ( loc_statusp( i, j, SEEN ))
			{
				if ( loc_statusp( i, j, SECRET ))
				{
					saveloc = Level->site[ i ][ j ].locchar;
					set_correct_city_wall( i, j, Level );
					Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
					Level->site[ i ][ j ].locchar = saveloc;
				}
				else
					Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
			}
		}
	}
	
	fclose( fd );
}

/* loads the star peak into Level*/
void load_speak( int empty, int populate )
{
	int i, j, safe = Player.alignment > 0;
	char site;
	unsigned short saveloc;
	FILE *fd;

	if ( empty )
	{
		mprint("The peak is now devoid of inhabitants and treasure.");
		morewait();
	}

	if ( !populate )
    	empty = TRUE;

	TempLevel = Level;
	if ( ok_to_free( TempLevel ))
	{
		free_level( TempLevel );
		TempLevel = NULL;
	}
	Level = ( (plv)checkmalloc( sizeof( levtype )));
	clear_level( Level );
	Level->environment = E_STARPEAK;
	strcpy( Str3, Omegalib );
	strcat( Str3, "speak.dat" );
	fd = checkfopen( Str3, "rb" );
	site = cryptkey( "speak.dat" );
	
	for( j = 0; j < LENGTH; j++ )
	{
		for( i = 0; i < WIDTH; i++ )
		{
			Level->site[ i ][ j ].lstatus = 0;
			Level->site[ i ][ j ].roomnumber = RS_STARPEAK;
			Level->site[ i ][ j ].p_locf = L_NO_OP;
			site = getc( fd ) ^ site;
			switch( site )
			{
				case 'S':
					Level->site[ i ][ j ].locchar = GROUND;
					Level->site[ i ][ j ].showchar = WALL_WHOLE;
					lset( i, j, SECRET );
					Level->site[ i ][ j ].roomnumber = RS_SECRETPASSAGE;
					break;
					
				case 'L':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
					{
						make_site_monster( i, j, ML10 + 2 ); /* lawbringer */
						if ( safe )
							m_status_reset( Level->site[ i ][ j ].creature, HOSTILE );
					}
					break;
					
				case 's':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
					{
						make_site_monster( i, j, ML4 + 12 ); /* servant of law */
						if ( safe )
							m_status_reset( Level->site[ i ][ j ].creature, HOSTILE );
					}
					break;
					
				case 'M':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
					{
						make_site_monster( i, j, -1 );
						if ( safe )
							m_status_reset( Level->site[ i ][ j ].creature, HOSTILE );
					}
					break;
					
				case '$':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
						make_site_treasure( i, j, 10 );
					break;
					
				case '7':
					if ( !empty )
						Level->site[ i ][ j ].locchar = V_PORTCULLIS;
					else
						Level->site[ i ][ j ].locchar = V_GROOVE;
					Level->site[ i ][ j ].p_locf = L_PORTCULLIS;
					break;
					
				case 'R':
					Level->site[ i ][ j ].locchar = GROUND;
					Level->site[ i ][ j ].p_locf = L_RAISE_PORTCULLIS;
					break;
					
				case '-':
					Level->site[ i ][ j ].locchar = MY_CLOSED_DOOR;
					break;
					
				case '|':
					Level->site[ i ][ j ].locchar = MY_OPEN_DOOR;
					break;
					
				case 'p':
					Level->site[ i ][ j ].locchar = V_GROOVE;
					Level->site[ i ][ j ].p_locf = L_PORTCULLIS;
					break;
					
				case 'T':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
						Level->site[ i ][ j ].p_locf = L_PORTCULLIS_TRAP;
					break;
					
				case 'X':
					Level->site[ i ][ j ].locchar = GROUND;
					Level->site[ i ][ j ].p_locf = L_TACTICAL_EXIT;
					break;
					
				case '#':
					Level->site[ i ][ j ].locchar = WALL_WHOLE;
					Level->site[ i ][ j ].aux = 150; 
					break;
					
				case '4':
					Level->site[i][j].locchar = MY_RUBBLE;
					Level->site[i][j].p_locf = L_RUBBLE;
					break;
					
				case '.':
					Level->site[i][j].locchar = GROUND;
					break;
			}
		}
		site = getc( fd ) ^ site;
	}
	
	for( j = 0; j < LENGTH; j++ )
	{
		for( i = 0; i < WIDTH; i++ )
		{
			switch( Level->site[ i ][ j ].locchar )
			{
				case WALL_WHOLE:
					set_correct_city_wall( i, j, Level );
					break;
					
				case POOL_WHOLE:
					set_correct_pool( i, j, Level );
					break;
					
				case HEDGE_WHOLE:
					set_correct_hedge( i, j, Level );
					break;
			}

			if ( loc_statusp( i, j, SEEN ))
			{
				if ( loc_statusp( i, j, SECRET ))
				{
					saveloc = Level->site[ i ][ j ].locchar;
					set_correct_city_wall( i, j, Level );
					Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
					Level->site[ i ][ j ].locchar = saveloc;
				}
				else
					Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
			}
		}
	}
	
	fclose( fd );
}

/* loads the magic isle into Level*/
void load_misle( int empty, int populate )
{
	int i, j;
	char site;
	unsigned short saveloc;
	FILE *fd;

	if ( empty )
	{
		mprint( "The isle is now devoid of inhabitants and treasure." );
		morewait();
	}

	if ( !populate )
		empty = TRUE;

	TempLevel = Level;
	if ( ok_to_free( TempLevel ))
	{
		free_level( TempLevel );
		TempLevel = NULL;
	}
	Level = ( (plv)checkmalloc( sizeof( levtype )));
	clear_level( Level );
	Level->environment = E_MAGIC_ISLE;
	strcpy( Str3, Omegalib );
	strcat( Str3, "misle.dat" );
	fd = checkfopen( Str3, "rb" );
	site = cryptkey( "misle.dat" );

	for( j = 0; j < LENGTH; j++ )
	{
		for( i = 0; i < WIDTH; i++ )
		{
			Level->site[ i ][ j ].lstatus = 0;
			Level->site[ i ][ j ].roomnumber = RS_MAGIC_ISLE;
			Level->site[ i ][ j ].p_locf = L_NO_OP;
			site = getc( fd ) ^ site;
			switch( site )
			{
				case 'E':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
						make_site_monster( i, j, ML10 + 1 ); /* eater of magic */
					break;
					
				case 'm':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
						make_site_monster( i, j, ML8 + 9 ); /* militant priest */
					break;

				case 'n':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !empty )
						make_site_monster( i, j, ML7 + 1 ); /* nazgul */
					break;

				case 'X':
					Level->site[ i ][ j ].locchar = GROUND;
					Level->site[ i ][ j ].p_locf = L_TACTICAL_EXIT;
					break;

				case '#':
					Level->site[ i ][ j ].locchar = WALL_WHOLE;
					Level->site[ i ][ j ].aux = 150; 
					break;

				case '4':
					Level->site[ i ][ j ].locchar = MY_RUBBLE;
					Level->site[ i ][ j ].p_locf = L_RUBBLE;
					break;

				case '~':
					Level->site[ i ][ j ].locchar = POOL_WHOLE;
					Level->site[ i ][ j ].p_locf = L_CHAOS;
					break;

				case '=':
					Level->site[ i ][ j ].locchar = POOL_WHOLE;
					Level->site[ i ][  j].p_locf = L_MAGIC_POOL;
					break;

				case '-':
					Level->site[ i ][ j ].locchar = MY_CLOSED_DOOR;
					break;

				case '|':
					Level->site[ i ][ j ].locchar = MY_OPEN_DOOR;
					break;

				case '.':
					Level->site[ i ][ j ].locchar = GROUND;
					break;
			}
		}
		site = getc( fd ) ^ site;
	}
	
	for( j = 0; j < LENGTH; j++ )
	{
		for( i = 0; i < WIDTH; i++ )
		{
			switch( Level->site[ i ][ j ].locchar )
			{
				case WALL_WHOLE:
					set_correct_city_wall( i, j, Level );
					break;
					
				case POOL_WHOLE:
					set_correct_pool( i, j, Level );
					break;
					
				case HEDGE_WHOLE:
					set_correct_hedge( i, j, Level );
					break;
			}

			if ( loc_statusp( i, j, SEEN ))
			{
				if ( loc_statusp( i, j, SECRET ))
				{
					saveloc = Level->site[ i ][ j ].locchar;
					set_correct_city_wall( i, j, Level );
					Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
					Level->site[ i ][ j ].locchar = saveloc;
				}
				else
					Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
			}
		}
	}
	fclose( fd );
}

/* loads a temple into Level*/
void load_temple( int deity, int populate )
{
	int i, j;
	char site;
	unsigned short saveloc;
	pml ml;
	FILE *fd;

	initrand( Current_Environment, deity );
	TempLevel = Level;
	if ( ok_to_free( TempLevel ))
	{
		free_level( TempLevel );
		TempLevel = NULL;
	}
	Level = ( (plv)checkmalloc( sizeof( levtype )));
	clear_level( Level );
	Level->environment = E_TEMPLE;
	strcpy( Str3, Omegalib );
	strcat( Str3, "temple.dat" );
	fd = checkfopen( Str3, "rb" );
	site = cryptkey( "temple.dat" );
	
	for( j = 0; j < LENGTH; j++ )
	{
		for( i = 0; i < WIDTH; i++ )
		{
			switch( deity )
			{
				case ODIN: Level->site[ i ][ j ].roomnumber = RS_ODIN; break;
				case SET: Level->site[ i ][ j ].roomnumber = RS_SET; break;
				case HECATE: Level->site[ i ][ j ].roomnumber = RS_HECATE; break;
				case ATHENA: Level->site[ i ][ j ].roomnumber = RS_ATHENA; break;
				case DRUID: Level->site[ i ][ j ].roomnumber = RS_DRUID; break;
				case DESTINY: Level->site[ i ][ j ].roomnumber = RS_DESTINY; break;
			}
			site = getc( fd ) ^ site;
			switch( site )
			{
				case '8':
					switch( deity )
					{
						case ODIN: Level->site[ i ][ j ].locchar = ALTAR_ODIN; break;
						case SET: Level->site[ i ][ j ].locchar = ALTAR_SET; break;
						case HECATE: Level->site[ i ][ j ].locchar = ALTAR_HECATE; break;
						case ATHENA: Level->site[ i ][ j ].locchar = ALTAR_ATHENA; break;
						case DRUID: Level->site[ i ][ j ].locchar = ALTAR_DRUID; break;
						case DESTINY: Level->site[ i ][ j ].locchar = ALTAR_DESTINY; break;
					}
					Level->site[ i ][ j ].p_locf = L_ALTAR;
					Level->site[ i ][ j ].aux = deity;
					break;

				case 'H':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( populate && ( !Player.patron ||
						 strcmp( Player.name, Priest[ Player.patron ] ) ||
						 Player.rank[ PRIESTHOOD ] != HIGHPRIEST ))
					{
						make_high_priest( i, j, deity );
					}
					break;
				
				case 'S':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( !Player.patron || strcmp( Player.name, Priest[ Player.patron ] ) ||
						 Player.rank[ PRIESTHOOD ] != HIGHPRIEST )
					{
						lset( i, j ,SECRET );
					}
					break;

				case 'W':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( deity != Player.patron && deity != DRUID )
						Level->site[ i ][ j ].p_locf = L_TEMPLE_WARNING;
					break;

				case 'm':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( populate )
						make_site_monster( i, j, ML8 + 9 ); /* militant priest */
					break;
					
				case 'd':
					Level->site[ i ][ j ].locchar = GROUND;
					if ( populate )
						make_site_monster( i, j, ML4 + 10 ); /* doberman death hound */
					break;

				case 'X':
					Level->site[ i ][ j ].locchar = GROUND;
					Level->site[ i ][ j ].p_locf = L_TACTICAL_EXIT;
					break;

				case '#':
					if ( deity != DRUID )
					{
						Level->site[ i ][ j ].locchar = WALL_WHOLE;
						Level->site[ i ][ j ].aux = 150; 
					}
					else
					{
						Level->site[ i ][ j ].locchar = HEDGE_WHOLE;
						Level->site[ i ][ j ].p_locf = L_HEDGE; 
					}
					break;

				case '.':
					Level->site[ i ][ j ].locchar = GROUND;
					break;

				case 'x':
					Level->site[ i ][ j ].locchar = GROUND;
					random_temple_site( i, j, deity, populate );
					break;

				case '?':
					if ( deity != DESTINY ) 
						Level->site[ i ][ j ].locchar = GROUND;
					else
					{
						Level->site[ i ][ j ].locchar = MY_ABYSS;
						Level->site[ i ][ j ].p_locf = L_ADEPT;
					}
					break;

				case '-':
					Level->site[ i ][ j ].locchar = MY_CLOSED_DOOR;
					break;

				case '|':
					Level->site[ i ][ j ].locchar = MY_OPEN_DOOR;
					break;
			}
		}
		site = getc( fd ) ^ site;
	}
	
	for( j = 0; j < LENGTH; j++ )
	{
		for( i = 0; i < WIDTH; i++ )
		{
			switch( Level->site[ i ][ j ].locchar )
			{
				case WALL_WHOLE:
					set_correct_city_wall( i, j, Level );
					break;
					
				case POOL_WHOLE:
					set_correct_pool( i, j, Level );
					break;
					
				case HEDGE_WHOLE:
					set_correct_hedge( i, j, Level );
					break;
			}

			if ( loc_statusp( i, j, SEEN ))
			{
				if ( loc_statusp( i, j, SECRET ))
				{
					saveloc = Level->site[ i ][ j ].locchar;
					set_correct_city_wall( i, j, Level );
					Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
					Level->site[ i ][ j ].locchar = saveloc;
				}
				else
					Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
			}
		}
	}
	
	/* Main Temple is peaceful for player of same sect, druids always peaceful. */
	if (( Player.patron == deity ) || ( deity == DRUID ))
		for( ml = Level->mlist; ml != NULL; ml = ml->next ) 
			m_status_reset( ml->m, HOSTILE );
	fclose( fd );
	initrand( -2, 0 );
}

void random_temple_site( int i, int j, int deity, int populate )
{
	switch( random_range( 12 ))
	{
		case 0:
			if ( populate )
				make_site_monster( i, j, ML0 + 1 );
			break; /* mendicant priest */

		case 1:
			Level->site[ i ][ j ].locchar = POOL_WHOLE;
			Level->site[ i ][ j ].p_locf = L_MAGIC_POOL;
			break;
			
		case 2:
			if ( populate )
				make_site_monster( i, j, ML7 + 14 );
			break; /* inner circle demon */

		case 3: 
			if ( populate )
				make_site_monster( i, j, ML6 + 11 );
			break; /* angel of apropriate sect */

		case 4: 
			if ( populate )
				make_site_monster( i, j, ML8 + 11 );
			break; /* high angel of apropriate sect */

		case 5: 
			if ( populate )
				make_site_monster( i, j, ML9 + 6 );
			break; /* archangel of apropriate sect */
	}
}

void make_high_priest( int i, int j, int deity )
{
	pml ml = ( (pml)checkmalloc( sizeof( mltype )));
	pmt m = ( (pmt)checkmalloc( sizeof( montype )));
	make_hiscore_npc( m, deity );
	m->x = i;
	m->y = j;
	Level->site[ i ][ j ].creature = m;
	ml->m = m;
	ml->next = Level->mlist;
	Level->mlist = ml;
}
