/*
** curses.h - Mac curses
** Copyright (c) 1996, Sheldon Simms
*/

#ifndef _CURSES_H_
#define _CURSES_H_

#include <stdio.h>
#include <stdarg.h>

#ifdef FALSE
#undef FALSE
#endif
#define FALSE 0

#ifdef TRUE
#undef TRUE
#endif
#define TRUE  1

#define ERR   0
#define OK    1

#define ACS_ULCORNER 0x80
#define ACS_LLCORNER 0x81
#define ACS_URCORNER 0x82
#define ACS_LRCORNER 0x83
#define ACS_RTEE     0x84
#define ACS_LTEE     0x85
#define ACS_BTEE     0x86
#define ACS_TTEE     0x87
#define ACS_HLINE    0x88
#define ACS_VLINE    0x89
#define ACS_PLUS     0x8a
#define ACS_S1       0x8b
#define ACS_S2       0x8c
#define ACS_S9       0x8c
#define ACS_DIAMOND  0x8d
#define ACS_CKBOARD  0x8e
#define ACS_DEGREE   0x8f
#define ACS_PLMINUS  0x90
#define ACS_BULLET   0x91
#define ACS_LARROW   0x92
#define ACS_RARROW   0x93
#define ACS_DARROW   0x94
#define ACS_UARROW   0x95
#define ACS_BOARD    0x96
#define ACS_LANTERN  0x97
#define ACS_BLOCK    0x98

#define KEY_BREAK    0x03
#define KEY_DOWN     0x1f
#define KEY_UP       0x1e
#define KEY_LEFT     0x1c
#define KEY_RIGHT    0x1d
#define KEY_ESCAPE   0x1b
#define KEY_DELETE   0x7f

typedef char bool;
typedef unsigned short chtype;

typedef struct window
{
	short curX, curY, lines, cols;
	short left, top, right, bottom;
	short type;
	unsigned int attrword;
	bool f_leave, f_scroll, f_clear, f_standout, f_nodelay;
	chtype **winBuf;
	short *begLine, *endLine;
	struct window *next;
} WINDOW;

extern int LINES, COLS;
extern WINDOW *stdscr, *curscr;

int initscr( void );
int endwin( void );
WINDOW *newwin( int lines, int cols, int uly, int ulx );
WINDOW *subwin( WINDOW *orig, int lines, int cols, int y, int x );
int mvwin( WINDOW *win, int y, int x );
int delwin( WINDOW *win );
int touchwin( WINDOW *win );
int touchline( WINDOW *win, int start, int count );
int overlay( WINDOW *srcwin, WINDOW *dstwin );
int overwrite( WINDOW *srcwin, WINDOW *dstwin );
int copywin( WINDOW *src, WINDOW *dst, int sminrow, int smincol, int dminrow,
				 int dmincol, int dmaxrow, int dmaxcol, int overlay );
int box( WINDOW *win, chtype vert, chtype horiz );
int scroll( WINDOW *win );
int scrollok( WINDOW *win, bool flag );
int leaveok( WINDOW *win, bool flag );
int clearok( WINDOW *win, bool flag );
int idlok( WINDOW *win, bool flag );
int nodelay( WINDOW *win, bool flag );

int cbreak( void );
int nocbreak( void );
int raw( void );
int noraw( void );
int echo( void );
int noecho( void );
int nl( void );
int nonl( void );
int baudrate( void );
char erasechar( void );
char killchar( void );
char *longname( void );
int wattrset( WINDOW *win, int attrs );
int beep( void );
int flash( void );
int keypad( WINDOW *win, bool flag );

int wstandout( WINDOW *win );
int wstandend( WINDOW *win );
int waddch( WINDOW *win, chtype c );
int waddstr( WINDOW *win, char *s );
int wclear( WINDOW *win );
int wclrtobot( WINDOW *win );
int wclrtoeol( WINDOW *win );
int werase( WINDOW *win );
int wdelch( WINDOW *win );
int wdeleteln( WINDOW *win );
chtype wgetch( WINDOW *win );
char *wgetstr( WINDOW *win, char *s );
int winsertln( WINDOW *win );
int winsch( WINDOW *win, chtype c );
int wmove( WINDOW *win, int y, int x );
int wnoutrefresh( WINDOW *win );
int doupdate( void );
int wrefresh( WINDOW *win );

int printw( const char *fstr, ... );
int wprintw( WINDOW *win, const char *fstr, ... );
int vwprintw( WINDOW *win, const char *fstr, va_list arg );
int mvprintw( int y, int x, const char *fstr, ... );
int mvwprintw( WINDOW *win, int y, int x, const char *fstr, ... );

/* generic macros */

#define getyx( w, y, x )		( y = w->curY, x = w->curX )
#define getbegyx( w, y, x )	( y = w->top, x = w->left )
#define getmaxyx( w, y, x )	( y = w->bottom, x = w->right )
#define winch( w )				( w->winBuf[ w->curY ][ w->curX ] & 0xff )
#define crmode()					cbreak()
#define nocrmode()				nocbreak()
#define gettmode()

/* stdscr macros */

#define standout()		wstandout( stdscr )
#define standend()		wstandend( stdscr )
#define addch( c )		waddch( stdscr, c )
#define addstr( s )		waddstr( stdscr, s )
#define attrset( a )		wattrset( stdscr, a )
#define clear()			wclear( stdscr )
#define clrtobot()		wclrtobot( stdscr )
#define clrtoeol()		wclrtoeol( stdscr )
#define delch()			wdelch( stdscr )
#define deleteln()		wdeleteln( stdscr )
#define erase()			werase( stdscr )
#define getch()			wgetch( stdscr )
#define getstr( s )		wgetstr( stdscr, s )
#define inch()				winch( stdscr )
#define insch( c )		winsch( stdscr, c )
#define insertln()		winsertln( stdscr )
#define move( y, x )		wmove( stdscr, y, x )
#define refresh()			wrefresh( stdscr )
#define standout()		wstandout( stdscr )
#define standend()		wstandend( stdscr )

/* move macros */

#define mvwaddch( w, y, x, c )	( wmove(w,y,x)==ERR ? ERR : waddch(w,c) )
#define mvwaddstr( w, y, x, s )	( wmove(w,y,x)==ERR ? ERR : waddstr(w,s) )
#define mvwdelch( w, y, x )		( wmove(w,y,x)==ERR ? ERR : wdelch(w) )
#define mvwgetch( w, y, x )		( wmove(w,y,x)==ERR ? ERR : wgetch(w) )
#define mvwgetstr( w, y, x, s )	( wmove(w,y,x)==ERR ? ERR : wgetstr(w,s) )
#define mvwinch( w, y, x )			( wmove(w,y,x)==ERR ? ERR : winch(w) )
#define mvwinsch( w, y, x, c )	( wmove(w,y,x)==ERR ? ERR : winsch(w,c) )

/* move stdscr macros */

#define mvaddch( y, x, c )		mvwaddch( stdscr, y, x, c )
#define mvaddstr( y, x, s )	mvwaddstr( stdscr, y, x, s )
#define mvdelch( y, x )			mvwdelch( stdscr, y, x )
#define mvgetch( y, x )			mvwgetch( stdscr, y, x )
#define mvgetstr( y, x, s )	mvwgetstr( stdscr, y, x, s )
#define mvinch( y, x )			mvwinch( stdscr, y, x )
#define mvinsch( y, x, c )		mvwinsch( stdscr, y, x, c )

#endif
