/* omega (c) 1987,1988,1989 by Laurence Raphael Brothers */
/* scr.c */
/* functions that use curses routines directly */
/* plus a few file i/o stuff */
/* also some in file.c */

#include <unistd.h>
#include "curses.h"
#include "glob.h"
#include "ochars.h"

void set_correct_hedge( int i, int j, plv Level );
void set_correct_city_wall( int i, int j, plv Level );
void set_correct_tile( int i, int j, plv Level );
void fixup_walls( int x, int y, plv Level );

void see_unlit_room( int x, int y, int roomno );
void tile_unlit_room( int x, int y, int roomno );
void light_unlit_room( int x, int y, int roomno );

extern int vault_changed, vault_open;

WINDOW *Bankw;

#ifdef EXCESSIVE_REDRAW
#define wclear werase
#endif

/* note these variables are not exported to other files */
WINDOW *Levelw, *Dataw, *Flagw, *Timew, *Menuw, *Locw, *Morew, *Phasew;
WINDOW *Comwin, *Msg1w, *Msg2w, *Msg3w, *Msgw;
WINDOW *Showline[ MAXITEMS ];

void phaseprint( void )
{
	werase( Phasew );
	wprintw( Phasew, "Moon's Phase:\n" );
	
	/*
	000000000011111111112222222222
	012345678901234567890123456789
	NccccccQggggggFFggggggQccccccN
	*/
	switch( Phase )
	{
		case 0: case 29:	wprintw( Phasew, "      New      " ); break;
		case 7:				wprintw( Phasew, " First Quarter " ); break;
		case 14: case 15:	wprintw( Phasew, "     Full      " ); break;
		case 22:			wprintw( Phasew, " Last Quarter  " ); break;
		
		default:
			if ( Phase < 7 )
				wprintw( Phasew, "Waxing Crescent" );
			else if ( Phase < 14 )
				wprintw( Phasew, "Waxing Gibbous " );
			else if ( Phase < 22 )
				wprintw( Phasew, "Waning Gibbous " );
			else
				wprintw( Phasew, "Waning Crescent" );
			break;
	}
	
	wrefresh( Phasew );
}

void show_screen( void )
{
	int i, j, top, bottom;
	int last_attr = 0, c;

	wclear( Levelw );
	top = ScreenOffset;
	bottom = ScreenOffset + ScreenLength;
	top = max( 0, top );
	bottom = min( bottom, LENGTH );
	if ( Current_Environment != E_COUNTRYSIDE ) 
		for ( j = top; j < bottom; j++ )
		{
			wmove( Levelw, screenmod( j ), 0 );
			for ( i = 0; i < WIDTH; i++ )
			{
				c = (( loc_statusp( i, j, SEEN )) ? getspot( i, j, FALSE ) : SPACE );
				if ( optionp( SHOW_COLOUR ) && ( c >> 8 ) != last_attr )
				{
					last_attr = c >> 8;
					wattrset( Levelw, last_attr );
				}
				waddch( Levelw, c );
				/* waddch( Levelw, c & 0xff ); */
			}
		}
	else for ( j = top; j < bottom; j++ )
		for ( i = 0; i < WIDTH; i++ )
		{
			wmove( Levelw, screenmod( j ), i );
			c = (( c_statusp( i, j, SEEN )) ? Country[ i ][ j ].current_terrain_type : SPACE );
			if ( optionp( SHOW_COLOUR ) && ( c >> 8 ) != last_attr )
			{
				last_attr = c >> 8;
				wattrset( Levelw, last_attr );
			}
			waddch( Levelw, c );
			/* waddch( Levelw, c & 0xff ); */
		}
	wrefresh( Levelw );
}

char mgetc( void )
{
	return( wgetch( Msgw ));
}

/* case insensitive mgetc -- sends uppercase to lowercase */
int mcigetc()
{
	int c;

	c = wgetch( Msgw );
	if (( c >= (int)'A' ) && ( c <= (int)'Z' ))
		return( c + (int)( 'a' - 'A' ));
	else
		return( c );
}

char menugetc( void )
{
	return( wgetch( Menuw ));
}

char lgetc( void )
{
	return( wgetch( Levelw ));
}

char ynq( void )
{
	char p = ' ';

	while (( p != 'n' ) && ( p != 'y' ) && ( p != 'q' ) && ( p != ESCAPE ))
		p = wgetch( Msgw );
	switch( p )
	{
		case 'y': wprintw( Msgw, "yes. " ); break;
		case 'n': wprintw( Msgw, "no. " ); break;
		case ESCAPE: p = 'q';
		case 'q': wprintw( Msgw, "quit. " ); break;
	}
	wrefresh( Msgw );
	return( p );
}

char ynq1( void )
{
	char p = ' ';

	while (( p != 'n' ) && ( p != 'y' ) && ( p != 'q' ) && ( p != ESCAPE ))
		p = wgetch( Msg1w );
	switch( p )
	{
		case 'y': wprintw( Msg1w, "yes. " ); break;
		case 'n': wprintw( Msg1w, "no. " ); break;
		case ESCAPE: p = 'q';
		case 'q': wprintw( Msg1w, "quit. " ); break;
	}
	wrefresh( Msg1w );
	return( p );
}

char ynq2( void )
{
	char p = ' ';

	while (( p != 'n' ) && ( p != 'y' ) && ( p != 'q' ) && ( p != ESCAPE ))
		p = wgetch( Msg2w );
	switch( p )
	{
		case 'y': wprintw( Msg2w, "yes. " ); break;
		case 'n': wprintw( Msg2w, "no. " ); break;
		case ESCAPE: p = 'q';
		case 'q': wprintw( Msg2w, "quit. " ); break;
	}
	wrefresh( Msg2w );
	return( p );
}
    
/* puts up a morewait to allow reading if anything in top two lines */
void checkclear( void )
{
	int x1, y, x2;

	getyx( Msg1w, x1, y );
	getyx( Msg2w, x2, y );  
	if (( x1 != 0 ) || ( x2 != 0 ))
	{
		morewait();
		wclear( Msg1w );
		wclear( Msg2w );
		wrefresh( Msg1w );
		wrefresh( Msg2w );
	}
}
  
/* for external call */
void clearmsg( void )
{
	wclear( Msg1w );
	wclear( Msg2w );
	wclear( Msg3w );
	Msgw = Msg1w;
	wrefresh( Msg1w );
	wrefresh( Msg2w );
	wrefresh( Msg3w );
}

void clearmsg3( void )
{
	wclear( Msg3w );
	wrefresh( Msg3w );
}

void clearmsg1( void )
{
	wclear( Msg1w );
	wclear( Msg2w );
	Msgw = Msg1w;
	wrefresh( Msg1w );
	wrefresh( Msg2w );
}

void erase_level( void )
{
	wclear( Levelw );
	wrefresh( Levelw );
}

/* direct print to first msg line */
void print1( char *s )
{
	if ( !gamestatusp( SUPPRESS_PRINTING ))
	{
		buffercycle( s );
		wclear( Msg1w );
		wprintw( Msg1w, s );
		wrefresh( Msg1w );
	}
}

/* for run on-messages -- print1 clears first.... */
void nprint1( char *s )
{
	if ( !gamestatusp( SUPPRESS_PRINTING ))
	{
		if ( bufferappend( s ))
		{
			wprintw( Msg1w, s );
			wrefresh( Msg1w );
		}
	}
}

/* direct print to second msg line */
void print2( char *s )
{
	if ( !gamestatusp( SUPPRESS_PRINTING ))
	{
		buffercycle( s );
		wclear( Msg2w );
		wprintw( Msg2w, s );
		wrefresh( Msg2w );
	}
}

/* for run on-messages -- print2 clears first.... */
void nprint2( char *s )
{
	if ( !gamestatusp( SUPPRESS_PRINTING ))
	{
		if ( bufferappend( s ))
		{
			wprintw( Msg2w, s );
			wrefresh( Msg2w );
		}
	}
}

/* msg line 3 is not part of the region that mprint or printm can reach */
/* typical use of print3 is for "you can't do that" type error messages */
void print3( char *s )
{
	if ( !gamestatusp( SUPPRESS_PRINTING ))
	{
		buffercycle( s );
		wclear( Msg3w );
		wprintw( Msg3w, s );
		wrefresh( Msg3w );
	}
}

/* for run on-messages -- print3 clears first.... */
void nprint3( char *s )
{
	if ( !gamestatusp( SUPPRESS_PRINTING ))
	{
		if ( bufferappend( s ))
		{
			wprintw( Msg3w, s );
			wrefresh( Msg3w );
		}
	}
}

/* prints wherever cursor is in window, but checks to see if
it should morewait and clear window */
void mprint( char *s )
{
	int x, y;
	int cycle;
	
	if ( !gamestatusp( SUPPRESS_PRINTING ))
	{
		getyx( Msgw, y, x );
		if ( COLS < 80 )
		{
			if ( Msgw == Msg3w )
			{
				if ( ( x + strlen( s )) >= ( WIDTH - 16 ) )
					cycle = TRUE;
				else
					cycle = FALSE;
			}
			else if ( x + strlen( s ) >= WIDTH )
				cycle = TRUE;
			else
				cycle = FALSE;
		}
		else if ( x + strlen( s ) >= WIDTH )
			cycle = TRUE;
		else
			cycle = FALSE;
			
		if ( cycle )
		{
			buffercycle( s );
			if ( Msgw == Msg1w )
			{
				wclear( Msg2w );
				Msgw = Msg2w;
			}
			else
			{
				morewait();
				wclear( Msg1w );
				wclear( Msg2w );
				wrefresh( Msg2w );
				Msgw = Msg1w;
			}
		}
		else if ( x > 0 )
			bufferappend( s );
		else
			buffercycle( s );
			
		wprintw( Msgw, s );
		waddch( Msgw, ' ' ); 
		wrefresh( Msgw );
	}
}

void title( void )
{
	showmotd();
	clear();
	touchwin( stdscr );
	refresh();  
	showscores();
}

/* blanks out ith line of Menuw or Levelw */
void hide_line( int i )
{
	wclear( Showline[ i ] );
	touchwin( Showline[ i ] );
	wrefresh( Showline[ i ] );
}

/* initialize, screen, windows */
void initgraf( void )
{
	int i;

	initscr();
	if ( LINES < 24 || COLS < 64 )
	{
		refresh();
		addstr( "Sorry, Omega cannot run on a screen this small." );
		addstr( "\n\nHit any key to quit\n" );
		refresh();
		getch();
		exit( 0 );
	}
	
	Msg1w = newwin( 1, COLS, 0, 0 );
	Msg2w = newwin( 1, COLS, 1, 0 );
	Msgw = Msg1w;
	
	for( i = 0; i < MAXITEMS; i++ )
	{
		Showline[ i ] = newwin( 1, 64, i + 3, 0 );
		wclear( Showline[ i ] );
	}
	
	if ( COLS < 80 )
	{
		ScreenLength = LINES - 11;
		Msg3w = newwin( 1, COLS - 16, 2, 0 );
		Morew = newwin( 1, 15, 2, 48 );
		Locw = newwin( 1, COLS, ScreenLength + 3, 0 );
		Timew = newwin( 1, 32, ScreenLength + 4, 0 );
		Phasew = newwin( 1, 32, ScreenLength + 4, 32 );
		Comwin = newwin( 1, 64, ScreenLength + 6, 0 );
		Dataw = newwin( 2, COLS, ScreenLength + 7, 0 );
		Flagw = newwin( 1, 64, ScreenLength + 10, 0 );
	}
	else
	{
		ScreenLength = LINES - 6;
		Msg3w = newwin( 1, COLS, 2, 0 );
		Morew = newwin( 1, 15, 3, 65 );
		Timew = newwin( 2, 15, 4, 65 );
		Phasew = newwin( 2, 15, 6, 65 );
		Flagw = newwin( 4, 15, 9, 65 );
		Comwin = newwin( 8, 15, 14, 65 );
		Locw = newwin( 1, 80, ScreenLength + 3, 0 );
		Dataw = newwin( 2, 80, ScreenLength + 4, 0 );
	}
	
	Bankw = newwin( 20, 40, ( ScreenLength - 20 ) / 2 + 3, 12 );
	leaveok( Bankw, TRUE );
	Levelw = newwin( ScreenLength, 64, 3, 0 );
	Menuw = newwin( ScreenLength, 64, 3, 0 );

	noecho();
	crmode();

	clear();
	touchwin( stdscr );
	title();
	clear();
	touchwin( stdscr );
	refresh();
}

int lastx = -1,lasty = -1;
void drawplayer( void )
{
	int c;

	if ( Current_Environment == E_COUNTRYSIDE )
	{
		if ( inbounds( lastx, lasty ) && !offscreen( lasty ))
		{
			wmove( Levelw, screenmod( lasty ), lastx );
			c = Country[ lastx ][ lasty ].current_terrain_type;
			if ( optionp( SHOW_COLOUR ))
				wattrset( Levelw, c >> 8 );
			waddch( Levelw, c );
			/* waddch( Levelw, ( c & 0xff )); */
		}
		wmove( Levelw, screenmod( Player.y ), Player.x );
		if ( optionp( SHOW_COLOUR ))
			wattrset( Levelw, PLAYER >> 8 );
		waddch( Levelw, C_PLAYER );
		/* waddch( Levelw, ( PLAYER & 0xff )); */
	}
	else
	{
		if ( inbounds( lastx, lasty ) && !offscreen( lasty ))
			plotspot( lastx, lasty, ( Player.status[ BLINDED ] > 0 ? FALSE : TRUE ));
		wmove( Levelw, screenmod( Player.y ), Player.x );
		if (( !Player.status[ INVISIBLE ] ) || Player.status[ TRUESIGHT ] )
		{
			if ( optionp( SHOW_COLOUR ))
				wattrset( Levelw, PLAYER >> 8 );
			waddch( Levelw, C_PLAYER );
			/* waddch( Levelw, ( PLAYER & 0xff )); */
		}
	}
	lastx = Player.x;
	lasty = Player.y;
}

void setlastxy( int new_x, int new_y ) /* used when changing environments */
{
	lastx = new_x;
	lasty = new_y;
}

int litroom( int x, int y )
{
	if ( Level->site[ x ][ y ].roomnumber < ROOMBASE )
		return( FALSE );
	else
		return( loc_statusp( x, y, LIT ) || Player.status[ ILLUMINATION ] );
}

void drawvision( int x, int y )
{
	static int oldx = -1, oldy = -1;
	int i, j, c;
	int see_radius;

	if ( Current_Environment != E_COUNTRYSIDE )
	{
		if ( Player.status[ BLINDED ] )
		{
			drawspot( oldx, oldy );
			drawspot( x, y );
			drawplayer();
		}
		else
		{
			if ( Player.status[ ILLUMINATION ] )
			{
				see_radius = 2;
			}
			else
			{
				see_radius = 1;
			}
			
			/* First tag new site and sites roughly within see_radius */
			/* spaces as having been seen. */
			for ( i = x - see_radius; i <= x + see_radius; i++ )
			{
				for ( j = y - see_radius; j <= y + see_radius; j++ )
				{
					if ( inbounds( i, j ) && view_los_p( Player.x, Player.y, i, j ) )
					{
						lset( i, j, SEEN );
					}
				}
			}
				
			/* Now make sure the tiles surrounding the new location */
			/* within see_radius + 1 spaces are set correctly in the */
			/* level->site[][] array based on what has now been seen. */
			see_radius++;
			for ( i = x - see_radius; i <= x + see_radius; i++ )
			{
				for ( j = y - see_radius; j <= y + see_radius; j++ )
				{
					if ( inbounds( i, j ) && view_los_p( Player.x, Player.y, i, j ) )
					{
						set_correct_tile( i, j, Level );
					}
				}
			}
				
			/* Now draw the tiles as they have just been set, but don't */
			/* draw tiles that have not yet been seen. */
			for ( i = x - see_radius; i <= x + see_radius; i++ )
			{
				for ( j = y - see_radius; j <= y + see_radius; j++ )
				{
					if ( inbounds( i, j ) )
					{
						if ( loc_statusp( i, j, SEEN ) )
						{
							c = getspot( i, j, FALSE );
							Level->site[ i ][ j ].showchar = c;
							putspot( i, j, c );
						}
					}
				}
			}
			
			drawmonsters( FALSE ); /* erase all monsters */
			drawmonsters( TRUE );  /* draw those now visible */
			
			drawplayer();
		}
		if (( !gamestatusp( FAST_MOVE )) || ( !optionp( JUMPMOVE )))
			omshowcursor( Player.x, Player.y );
		oldx = x;
		oldy = y;
	}
	else
	{
		for ( i = -1; i < 2; i++ )
		{
			for ( j = -1; j < 2; j++ )
			{
				if ( inbounds( x + i, y + j ))
				{
					c_set( x + i, y + j, SEEN );
					if ( !offscreen( y + j ))
					{
						wmove( Levelw, screenmod( y + j ), x + i );
						c = Country[ x + i ][ y + j ].current_terrain_type;
						waddch( Levelw, c );
					}
				}
			}
		}
		drawplayer();
		omshowcursor( Player.x, Player.y );
	}
}

void omshowcursor( int x, int y )
{
	if ( !offscreen( y ))
	{
		wmove( Levelw, screenmod( y ), x );
		wrefresh( Levelw );
	}
}

void levelrefresh( void )
{
	wrefresh( Levelw );
}

/* draws a particular spot under if in line-of-sight */
void drawspot( int x, int y )
{
	short c;

	if ( inbounds( x, y ) )
	{
		c = getspot( x, y, FALSE );
		if ( c != Level->site[ x ][ y ].showchar )
		{
			if ( view_los_p( Player.x, Player.y, x, y ))
			{
				lset( x, y, SEEN );
				Level->site[ x ][ y ].showchar = c;
				putspot( x, y, c );
			}
		}
	}
}

/* draws a particular spot regardless of line-of-sight */
void dodrawspot( int x, int y )
{
	short c;

	if ( inbounds( x, y ))
	{
		c = getspot( x, y, FALSE );
		lset( x, y, SEEN );
		Level->site[ x ][ y ].showchar = c;
		putspot( x, y, c );
	}
}

/* blank out a spot regardless */
void blotspot( int i, int j )
{
	if ( inbounds( i, j ))
	{
		lreset( i, j, SEEN );
		Level->site[ i ][ j ].showchar = SPACE;
		if ( !offscreen( j ) )
		{
			wmove( Levelw, screenmod( j ), i );
			wattrset( Levelw, SPACE >> 8 );
			waddch( Levelw, SPACE );
		}
	}
}

/* for displaying activity specifically at some point */
void plotspot( int x, int y, int showmonster )
{
	if ( loc_statusp( x, y, SEEN ))
		putspot( x, y, getspot( x, y, showmonster ));
	else 
		putspot( x, y, SPACE );
}

/* Puts c at x,y on screen. No fuss, no bother. */
void putspot( int x, int y, short c )
{
	if ( !offscreen( y ))
	{
		wmove( Levelw, screenmod( y ), x );
		waddch( Levelw, c );
	}
}

/* regardless of line of sight, etc, draw a monster */
void plotmon( struct monster *m )
{
	if ( !offscreen( m->y ) )
	{
		wmove( Levelw, screenmod( m->y ), m->x );
		waddch( Levelw, m->monchar );
	}
}
  
/* if display, displays monsters, otherwise erases them */
void drawmonsters( int display )
{
	pml ml;

	for ( ml = Level->mlist; ml != NULL; ml = ml->next )
	{
		if ( ml->m->hp > 0 )
		{
			if ( display )
			{
				if ( view_los_p( Player.x, Player.y, ml->m->x, ml->m->y ))
				{
					if ( Player.status[ TRUESIGHT ] || ( !m_statusp( ml->m, M_INVISIBLE )))
					{
						putspot( ml->m->x, ml->m->y, ml->m->monchar );
					}
				}
			}
			else
				erase_monster( ml->m );
		}
	}
}

/* replace monster with what would be displayed if monster weren't there */
void erase_monster( struct monster *m )
{
	if ( loc_statusp( m->x, m->y, SEEN ))
		putspot( m->x, m->y, getspot( m->x, m->y, FALSE ));
	else
		blotspot( m->x, m->y );
}

/* find apt char to display at some location */
short getspot( int x, int y, int showmonster )
{
	short saveloc, showchar, hedgecount = 0;
	int i, j;
	
	if ( loc_statusp( x, y, SECRET ))
	{
		/* See if this is the secret passage in woodhenge */
		/* maybe I'll find a better way to do this. */
		for ( j = y-1; j < y+2; j+=2 )
			for ( i = x-1; i < x+2; i++ )
				if ( inbounds( i, j ))
				{
					if ( Level->site[ i ][ j ].locchar >= HEDGE_FIRST &&
						 Level->site[ i ][ j ].locchar <= HEDGE_LAST )
					{
						hedgecount++;
					}
				}
		saveloc = Level->site[ x ][ y ].locchar;
		if ( hedgecount >= 5 )
			set_correct_hedge( x, y, Level );
		else
			set_correct_city_wall( x, y, Level );
		showchar = Level->site[ x ][ y ].locchar;
		Level->site[ x ][ y ].locchar = saveloc;
		return showchar;
		/* return( WALL_WHOLE ); */
	}
	else if (( Level->site[ x ][ y ].locchar == WATER ) ||
			 ( Level->site[ x ][ y ].locchar >= POOL_FIRST &&
			   Level->site[ x ][ y ].locchar <= POOL_LAST ))
	{
		if ( Level->site[ x ][ y ].creature == NULL ) 
			return( Level->site[ x ][ y ].locchar );
		else if ( m_statusp( Level->site[ x ][ y ].creature, SWIMMING ))
			return( Level->site[ x ][ y ].locchar );
		else if ( showmonster )
			return( Level->site[ x ][ y ].creature->monchar );
		else
			return( Level->site[ x ][ y ].locchar );
	}
	else if (( Level->site[ x ][ y ].locchar == RUBBLE ) ||
			 ( Level->site[ x ][ y ].locchar == MY_RUBBLE ) ||
			 ( Level->site[ x ][ y ].locchar >= HEDGE_FIRST &&
			   Level->site[ x ][ y ].locchar <= HEDGE_LAST ))
	{
		/* rubble and hedge don't show items on their location */
		if ( showmonster && ( Level->site[ x ][ y ].creature != NULL ))
		{
			if (( m_statusp( Level->site[ x ][ y ].creature, M_INVISIBLE )) &&
				( !Player.status[ TRUESIGHT ] ))
			{
				return( getspot( x, y, FALSE ));
			}
			else
				return( Level->site[ x ][ y ].creature->monchar );
		}
		else 
			return( Level->site[ x ][ y ].locchar );
	}
	else switch ( Level->site[ x ][ y ].locchar )
	{
		/* these sites never show anything but their location char's */
		case MY_CLOSED_DOOR:
		case MY_LAVA:
		case MY_FIRE:
		case MY_ABYSS:
		case CLOSED_DOOR:
		case LAVA:
		case FIRE:
		case ABYSS:
			return( Level->site[ x ][ y ].locchar );
				
		/* everywhere else, first try to show monster, next show items, next show
		location char */
		default:
			if ( showmonster && ( Level->site[ x ][ y ].creature != NULL ))
			{
				if (( m_statusp( Level->site[ x ][ y ].creature, M_INVISIBLE )) &&
					( !Player.status[ TRUESIGHT ] ))
				{
					return( getspot( x, y, FALSE ));
				}
				else
					return( Level->site[ x ][ y ].creature->monchar );
			}
			else if ( Level->site[ x ][ y ].things != NULL )
			{
				if ( Level->site[ x ][ y ].things->next != NULL )
					return( C_JUNK );
				else
					return( Level->site[ x ][ y ].things->thing->objchar );
			}
			else
				return( Level->site[ x ][ y ].locchar );
	}
}

void commanderror( void )
{
	wclear( Msg3w );
	wprintw( Msg3w, "%c : unknown command", Cmd );
	wrefresh( Msg3w );
}

void timeprint( void )
{
	wclear( Timew );
	wprintw( Timew, "%d:%d", showhour(), showminute() );
	if ( showminute() == 0 )
		waddch( Timew, '0' );
		
	if ( COLS < 80 )
		wprintw( Timew, hour() > 11 ? " PM " : " AM " );
	else
		wprintw( Timew, hour() > 11 ? " PM \n" : " AM \n" );
		
	wprintw( Timew, month() );
	wprintw( Timew, " the %d", day() );
	wprintw( Timew, ordinal( day() ));
	wrefresh( Timew );
	
	if (( vault_open == TRUE ) && ( Current_Environment == E_CITY ) && ( hour() != 23 ))
	{
		set_correct_city_wall( 12, 56, Level );
		if ( vault_changed )
			fixup_walls( 12, 56, Level );
		plotspot( 12, 56, FALSE );
		vault_open = FALSE;
	}
}

void comwinprint( void )
{
	wclear( Comwin );
	if ( COLS < 80 )
	{
		wprintw( Comwin, "Hit:%d Dmg:%d Def:%d Arm:%d Spd:%d.%d",
				 Player.hit, Player.dmg, Player.defense, Player.absorption,
				 5 / Player.speed, 500 / Player.speed % 100 );
	}
	else
	{
		wprintw( Comwin, "Hit: %d  \n", Player.hit );
		wprintw( Comwin, "Dmg: %d  \n", Player.dmg );
		wprintw( Comwin, "Def: %d  \n", Player.defense );
		wprintw( Comwin, "Arm: %d  \n", Player.absorption );
		wprintw( Comwin, "Spd: %d.%d  \n", 5 / Player.speed, 500 / Player.speed % 100 );
	}
	wrefresh( Comwin );
}

void dataprint( void )
{
	wclear( Dataw );
	wprintw( Dataw, "HP:%d/%d MANA:%ld/%ld AU:%ld LEVEL:%d/%ld CARRY:%d/%d \n",
			 Player.hp, Player.maxhp, Player.mana, Player.maxmana, Player.cash,
			 Player.level, Player.xp, Player.itemweight, Player.maxweight );
	wprintw( Dataw,"STR:%d/%d CON:%d/%d DEX:%d/%d AGI:%d/%d INT:%d/%d POW:%d/%d   ",
			 Player.str, Player.maxstr, Player.con, Player.maxcon,
			 Player.dex, Player.maxdex, Player.agi, Player.maxagi,
			 Player.iq, Player.maxiq, Player.pow, Player.maxpow );
	wrefresh( Dataw );
}

/* redraw everything currently displayed */
void redraw( void )
{
	touchwin( curscr );
	wrefresh( curscr );
}  

/* redraw each permanent window */
void xredraw( void )
{
	touchwin( Msgw );
	touchwin( Msg3w );
	touchwin( Levelw );
	touchwin( Timew );
	touchwin( Flagw );
	touchwin( Dataw );
	touchwin( Locw );
	touchwin( Morew );
	touchwin( Phasew );
	touchwin( Comwin ); 
	wrefresh( Msgw );
	wrefresh( Msg3w );
	wrefresh( Levelw );
	wrefresh( Timew );
	wrefresh( Flagw );
	wrefresh( Dataw );
	wrefresh( Locw );
	wrefresh( Morew );
	wrefresh( Phasew );
	wrefresh( Comwin );
}

void menuaddch( char c )
{
	waddch( Menuw, c );
	wrefresh( Menuw );
}

void morewait( void )
{
	int display = TRUE;
	char c;

	if ( gamestatusp( SUPPRESS_PRINTING ))
		return;
	do {
		wclear( Morew );
		if ( display )
			wprintw( Morew, "***  MORE  ***" );
		else
			wprintw( Morew, "+++  MORE  +++" );
		display = !display;
		wrefresh( Morew );
		c = wgetch( Msgw );
	} while (( c != ' ' ) && ( c != RETURN ));
	wclear( Morew );
	wrefresh( Morew );
}

int stillonblock( void )
{
	int display = TRUE;
	char c;

	do {
		wclear( Morew );
		if ( display )
			wprintw( Morew, "<<<STAY?>>>" );
		else
			wprintw( Morew, ">>>STAY?<<<" );
		display = !display;
		wrefresh( Morew );
		c = wgetch( Msgw );
	} while (( c != ' ' ) && ( c != ESCAPE ));
	wclear( Morew );
	wrefresh( Morew );
	return ( c == ' ' );
}

void menuclear( void )
{
	wclear( Menuw );
	touchwin( Menuw );
	wrefresh( Menuw );
}

void menuspellprint( int i )
{
	int x, y;

	getyx( Menuw, y, x );
	if ( y >= ScreenLength - 2 )
	{
		wrefresh( Menuw );
		morewait();
		wclear( Menuw );
		touchwin( Menuw );
	}
	wprintw( Menuw, spellid( i ));
	wprintw( Menuw, "(%d)\n", Spells[ i ].powerdrain );
}  

void menuprint( char *s )
{
	int x, y;

	getyx( Menuw, y, x );
	if ( y >= ScreenLength - 2 )
	{
		wrefresh( Menuw );
		morewait();
		wclear( Menuw );
		touchwin( Menuw );
	}
	wprintw( Menuw, s );
}

void showmenu( void )
{
	wrefresh( Menuw );
}

void endgraf( void )
{
	clear();
	touchwin( stdscr );
	refresh();
	endwin();
}

void plotchar( short pyx, int x, int y )
{
	if ( !offscreen( y ))
	{
		wmove( Levelw, screenmod( y ), x );
		if ( optionp( SHOW_COLOUR ))
			wattrset( Levelw, pyx >> 8 );
		waddch( Levelw, pyx );
		/* waddch( Levelw, ( pyx & 0xff )); */
		wrefresh( Levelw );
	}
}

void draw_explosion( short pyx, int x, int y )
{
	int i, j;
  
	for( j = 0; j < 3; j++ )
	{
		for( i = 0; i < 9; i++ ) 
			plotchar( pyx, x + Dirs[ 0 ][ i ], y + Dirs[ 1 ][ i ] );
		usleep( 150000 );
		for( i = 0; i < 9; i++ )
			plotchar( SPACE, x + Dirs[ 0 ][ i ], y + Dirs[ 1 ][ i ] );
		usleep( 150000 );
	}
	for( i = 0; i < 9; i++ ) 
		plotspot( x + Dirs[ 0 ][ i ], y + Dirs[ 1 ][ i ], TRUE );
	wrefresh( Levelw );
}

char *msgscanstring( void )
{
	static char instring[ 80 ], byte = 'x';
	int i = 0;

	instring[ 0 ] = 0;
	byte = mgetc();
	while ( byte != '\n' )
	{
		if (( byte == 8 ) || ( byte == 127 )) /* ^h or delete */
		{
			if ( i > 0)
			{ 
				i--;
				dobackspace();
			}
			instring[ i ] = 0;
		}
		else
		{
			instring[ i ] = byte;
			waddch( Msgw, byte );
			wrefresh( Msgw );
			i++;
			instring[ i ] = 0;
		}
		byte = mgetc();
	} 
	return( instring );
}

void locprint( char *s )
{
	wclear( Locw );
	wprintw( Locw, s );
	wrefresh( Locw );
}

/* draw everything whether visible or not */
void drawscreen( void )
{
	int i, j;
	short c;

	if ( Current_Environment == E_COUNTRYSIDE )
		for ( i = 0; i < WIDTH; i++ )
			for( j = 0; j < LENGTH; j++ )
				c_set( i, j, SEEN );
	else
	{
		for ( i = 0; i < WIDTH; i++ )
			for( j = 0; j < LENGTH; j++ )
				lset( i, j, SEEN );

		for ( i = 0; i < WIDTH; i++ )
			for( j = 0; j < LENGTH; j++ )
			{
				c = Level->site[ i ][ j ].locchar;
				if ( c >= WALL_FIRST && c <= WALL_LAST )
					set_correct_city_wall( i, j, Level );
			}
	}

	if ( Current_Environment == E_CITY )
		for ( i = 0; i < NUMCITYSITES; i++ )
			CitySiteList[ i ][ 0 ] = 1;
	show_screen();
}

/*selects a number up to range */
int getnumber( int range )
{
	int done = FALSE, value = 1;
	int atom;

	if ( range == 1 )
		return( 1 );
	else while ( !done )
	{
		clearmsg();
		wprintw( Msg1w, "How many? Change with < or >, ESCAPE to select:" );
		mnumprint( value );
		atom = mcigetc();
		switch (atom)
		{
			case '>':
			case 'k':
			case KEY_UP:
				if ( value < range )
					value++;
				break;

			case '<':
			case 'j':
			case KEY_DOWN:
				if ( value > 1 )
					value--;
				break;

			case ESCAPE:
				done = TRUE;
				break;
		}
	}
	return( value );
}

/* reads a positive number up to 999999 */
long parsenum( void )
{
	int number[ 8 ];
	int place = -1;
	int i, x, y, mult = 1;
	long num = 0;
	char byte = ' ';

	while (( byte != ESCAPE ) && ( byte != '\n' ))
	{
		byte = mgetc();
		if (( byte == BACKSPACE ) || ( byte == DELETE ))
		{
			if ( place > -1 )
			{
				number[ place ] = 0;
				place--;
				getyx( Msgw, y, x );
				wmove( Msgw, y, x - 1 );
				waddch( Msgw, ' ' );
				wmove( Msgw, y, x - 1 );
				wrefresh( Msgw );
			}
		}
		else if (( byte <= '9' ) && ( byte >= '0' ) && ( place < 7 ))
		{
			place++;
			number[ place ] = byte;
			waddch( Msgw, byte );
			wrefresh( Msgw );
		}
	}
	waddch( Msgw, ' ' );
	if ( byte == ESCAPE )
		return( ABORT );
	else
	{
		for ( i = place; i >= 0; i-- )
		{
			num += mult * ( number[ i ] - '0' );
			mult *= 10;
		}
		return( num );
	}
}

void maddch( char c )
{
	waddch( Msgw, c );
	wrefresh( Msgw );
}

void display_death( char *source )
{
	clear();
	touchwin( stdscr );
	printw( "\n\n\n\n" );
	printw( "Requiescat In Pace, " );
	printw( Player.name );
	printw( " (%ld points)", calc_points() );
	strcpy( Str4, "Killed by " );
	strcat( Str4, source );
	printw( "\n" );
	printw( Str4 );
	printw( "." );
	printw( "\n\n\n\n\nHit 'c' to continue." );
	refresh();
	while ( wgetch( stdscr ) != 'c' )
		;
	clear();
	touchwin( stdscr );
	refresh();
	extendlog( Str4, DEAD );
}

void display_win( void )
{
	clear();
	touchwin( stdscr );
	printw( "\n\n\n\n" );
	printw( Player.name );
	if ( Player.rank[ ADEPT ] )
	{
		printw( " is a total master of omega with %ld points!", FixedPoints );
		strcpy( Str4, "A total master of omega" );
	}
	else
	{
		strcpy( Str4, "retired a winner" );
		printw( " triumphed in omega with %ld points!", calc_points() );
	}
	printw( "\n\n\n\n\nHit 'c' to continue." );
	refresh();
	while ( wgetch( stdscr ) != 'c' )
		;
	clear();
	touchwin( stdscr );
	refresh();
	if ( Player.rank[ ADEPT ] )
		extendlog( Str4, BIGWIN );
	else
		extendlog( Str4, WIN );
}

void display_quit( void )
{
	clear();
	touchwin( stdscr );
	printw( "\n\n\n\n" );
	printw( Player.name );
	strcpy( Str4, "A quitter." );
	printw( " wimped out with %ld points!", calc_points() );
	printw( "\n\n\n\n\nHit 'c' to continue." );
	refresh();
	while ( wgetch( stdscr ) != 'c' )
		;
	clear();
	touchwin( stdscr );
	refresh();
	extendlog( Str4, QUIT );
}

void display_bigwin( void )
{
	clear();
	touchwin( stdscr );
	printw( "\n\n\n\n" );
	printw( Player.name );
	strcpy( Str4, "retired, an Adept of Omega." );
	printw( " retired, an Adept of Omega with %ld points!", FixedPoints );
	printw( "\n\n\n\n\nHit 'c' to continue." );
	refresh();
	while ( wgetch( stdscr ) != 'c' )
		;
	clear();
	touchwin( stdscr );
	refresh();
	extendlog( Str4, BIGWIN );
}

void mnumprint( int n )
{
	char numstr[ 20 ];

	sprintf( numstr, "%d", n );
	bufferappend( numstr );
	wprintw( Msgw, "%d", n );
	wrefresh( Msgw );
}

void mlongprint( long n )
{
	char numstr[ 20 ];

	sprintf( numstr, "%ld", n );
	bufferappend( numstr );
	wprintw( Msgw, "%ld", n );
	wrefresh( Msgw );
}

void menunumprint( int n )
{
	int x, y;

	getyx( Menuw, y, x );
	if ( y >= ScreenLength - 2 )
	{
		wrefresh( Menuw );
		morewait();
		wclear( Menuw );
		touchwin( Menuw );
	}
	wprintw( Menuw, "%d", n );
}

void menulongprint( long n )
{
	int x, y;

	getyx( Menuw, y, x );
	if ( y >= ScreenLength - 2 )
	{
		wrefresh( Menuw );
		morewait();
		wclear( Menuw );
		touchwin( Menuw );
	}
	wprintw( Menuw, "%ld", n );
}

void dobackspace( void )
{
	int x, y;

	getyx( Msgw, y, x );
	if ( x > 0 )
	{
		waddch( Msgw, ' ' );
		wmove( Msgw, y, x - 1 );
		waddch( Msgw, ' ' );
		wmove( Msgw, y, x - 1 );
	}
	wrefresh( Msgw );
}

void showflags( void )
{
	phaseprint();
	wclear( Flagw );
	
	if ( COLS < 80 )
	{
		if ( Player.food < 0 )
			wprintw( Flagw, "Starving        " );
		else if ( Player.food <= 3 )
			wprintw( Flagw, "Weak            " );
		else if ( Player.food <= 10 )
			wprintw( Flagw, "Ravenous        " );
		else if ( Player.food <= 20 )
			wprintw( Flagw, "Hungry          " );
		else if ( Player.food <= 30 )
			wprintw( Flagw, "A mite peckish  " );
		else if ( Player.food <= 36 )
			wprintw( Flagw, "Content         " );
		else if ( Player.food <= 44 )
			wprintw( Flagw, "Satiated        " );
		else
			wprintw( Flagw, "Bloated         " );
			
		if ( Player.status[ POISONED ] > 0 )
			wprintw( Flagw, "Poisoned        " );
		else
			wprintw( Flagw, "Vigorous        " );
			
		if ( Player.status[ DISEASED ] > 0)
			wprintw( Flagw, "Diseased        " );
		else
			wprintw( Flagw, "Healthy         " );
			
		if ( gamestatusp( MOUNTED ))
			wprintw( Flagw, "Mounted" );
		else if ( Player.status[ LEVITATING ] )
			wprintw( Flagw, "Levitating" );
		else
			wprintw( Flagw, "Afoot" );
	}
	else
	{
		if ( Player.food < 0 )
			wprintw( Flagw, "Starving\n" );
		else if ( Player.food <= 3 )
			wprintw( Flagw, "Weak\n" );
		else if ( Player.food <= 10 )
			wprintw( Flagw, "Ravenous\n" );
		else if ( Player.food <= 20 )
			wprintw( Flagw, "Hungry\n" );
		else if ( Player.food <= 30 )
			wprintw( Flagw, "A mite peckish\n" );
		else if ( Player.food <= 36 )
			wprintw( Flagw, "Content\n" );
		else if ( Player.food <= 44 )
			wprintw( Flagw, "Satiated\n" );
		else
			wprintw( Flagw, "Bloated\n" );


		if ( Player.status[ POISONED ] > 0 )
			wprintw( Flagw, "Poisoned\n" );
		else
			wprintw( Flagw, "Vigorous\n" );
	
		if ( Player.status[ DISEASED ] > 0)
			wprintw( Flagw, "Diseased\n" );
		else
			wprintw( Flagw, "Healthy\n" );
	
		if ( gamestatusp( MOUNTED ))
			wprintw( Flagw, "Mounted\n" );
		else if ( Player.status[ LEVITATING ] )
			wprintw( Flagw, "Levitating\n" );
		else
			wprintw( Flagw, "Afoot\n" );
	}

	wrefresh( Flagw );
}

void drawomega( void )
{
	int i;

	clear();
	touchwin( stdscr );
	for(i = 0; i < 7; i++ )
	{
		move( 1, 1 );
		if ( optionp( SHOW_COLOUR ))
			wattrset( stdscr, COL_LIGHT_BLUE >> 8 );
		printw( "\n\n\n" );
		printw( "\n                                    *****" );
		printw( "\n                               ******   ******" );
		printw( "\n                             ***             ***" );
		printw( "\n                           ***                 ***" );
		printw( "\n                          **                     **" );
		printw( "\n                         ***                     ***" );
		printw( "\n                         **                       **" );
		printw( "\n                         **                       **" );
		printw( "\n                         ***                     ***" );
		printw( "\n                          ***                   ***" );
		printw( "\n                            **                 **" );
		printw( "\n                       *   ***                ***   *" );
		printw( "\n                        ****                    ****" );
		refresh();
		usleep( 200000 );
		move( 1, 1 );
		if ( optionp( SHOW_COLOUR ))
			wattrset( stdscr, COL_CYAN >> 8 );
		printw( "\n\n\n" );
		printw( "\n                                    +++++" );
		printw( "\n                               ++++++   ++++++" );
		printw( "\n                             +++             +++" );
		printw( "\n                           +++                 +++" );
		printw( "\n                          ++                     ++" );
		printw( "\n                         +++                     +++" );
		printw( "\n                         ++                       ++" );
		printw( "\n                         ++                       ++" );
		printw( "\n                         +++                     +++" );
		printw( "\n                          +++                   +++" );
		printw( "\n                            ++                 ++" );
		printw( "\n                       +   +++                +++   +" );
		printw( "\n                        ++++                    ++++" );
		refresh();
		usleep( 200000 );
		move( 1, 1 );
		if ( optionp( SHOW_COLOUR ))
			wattrset( stdscr, COL_BLUE >> 8 );
		printw( "\n\n\n" );
		printw( "\n                                    ....." );
		printw( "\n                               ......   ......" );
		printw( "\n                             ...             ..." );
		printw( "\n                           ...                 ..." );
		printw( "\n                          ..                     .." );
		printw( "\n                         ...                     ..." );
		printw( "\n                         ..                       .." );
		printw( "\n                         ..                       .." );
		printw( "\n                         ...                     ..." );
		printw( "\n                          ...                   ..." );
		printw( "\n                            ..                 .." );
		printw( "\n                       .   ...                ...   ." );
		printw( "\n                        ....                    ...." );
		refresh();
		usleep( 200000 );
	}
	wattrset( stdscr, COL_WHITE >> 8 );
}

/* y is an absolute coordinate */
/* ScreenOffset is the upper left hand corner of the current screen
   in absolute coordinates */

void screencheck( int y )
{
	if ((( y - ScreenOffset ) < ( ScreenLength / 8 )) ||
		(( y - ScreenOffset ) > ( 7 * ScreenLength / 8 )))
	{
		ScreenOffset = y - ( ScreenLength / 2 );
		show_screen();
		if ( Current_Environment != E_COUNTRYSIDE ) 
			drawmonsters( TRUE );
		if ( !offscreen( Player.y ))
			drawplayer();
	}
}

void spreadroomlight( int x, int y, int roomno )
{
	/* First make all spaces in this room seen */
	see_unlit_room( x, y, roomno );
	
	/* Then set correct tile for each space in the room */
	tile_unlit_room( x, y, roomno );
	
	/* Now light the room, thus drawing it lit */
	light_unlit_room( x, y, roomno );
}

void see_unlit_room( int x, int y, int roomno )
{
	int i, j;
	
	if ( !inbounds( x, y ) )
		return;
		
	if ( loc_statusp( x, y, LIT ) )
		return;
		
	if ( Level->site[ x ][ y ].roomnumber != roomno )
		return;
	
	for ( i = x - 1; i <= x + 1; i++ )
	{
		for ( j = y - 1; j <= y + 1; j++ )
		{
			if ( inbounds( i, j ) )
			{
				lset( i, j, SEEN );
			}
		}
	}
	
	/* lset( x, y, SEEN ); */
				
	/* keeping track of where we've been */
	lset( x, y, LIT );
	
	see_unlit_room( x + 1, y, roomno );
	see_unlit_room( x, y + 1, roomno );
	see_unlit_room( x - 1, y, roomno );
	see_unlit_room( x, y - 1, roomno );
}

void tile_unlit_room( int x, int y, int roomno )
{
	int i, j;
	
	if ( !inbounds( x, y ) )
		return;
		
	if ( !loc_statusp( x, y, LIT ) )
		return;
		
	if ( Level->site[ x ][ y ].roomnumber != roomno )
		return;
	
	for ( i = x - 1; i <= x + 1; i++ )
	{
		for ( j = y - 1; j <= y + 1; j++ )
		{
			if ( inbounds( i, j ) )
			{
				set_correct_tile( i, j, Level );
			}
		}
	}
	
	/* set_correct_tile( x, y, Level ); */
	
	/* keeping track of where we've been */
	lreset( x, y, LIT );
	
	tile_unlit_room( x + 1, y, roomno );
	tile_unlit_room( x, y + 1, roomno );
	tile_unlit_room( x - 1, y, roomno );
	tile_unlit_room( x, y - 1, roomno );
}

void light_unlit_room( int x, int y, int roomno )
{
	short c;
	int i, j;
	
	if ( !inbounds( x, y ) )
		return;
		
	if ( loc_statusp( x, y, LIT ) )
		return;
		
	if ( Level->site[ x ][ y ].roomnumber != roomno )
		return;
	
	for ( i = x - 1; i <= x + 1; i++ )
	{
		for ( j = y - 1; j <= y + 1; j++ )
		{
			if ( inbounds( i, j ) )
			{
				c = getspot( i, j, FALSE );
				Level->site[ i ][ j ].showchar = c;
				putspot( i, j, c );
			}
		}
	}
	
	/*
	c = getspot( x, y, FALSE );
	Level->site[ x ][ y ].showchar = c;
	putspot( x, y, c );
	*/
	
	lset( x, y, LIT );
	lset( x, y, CHANGED );
	
	light_unlit_room( x + 1, y, roomno );
	light_unlit_room( x, y + 1, roomno );
	light_unlit_room( x - 1, y, roomno );
	light_unlit_room( x, y - 1, roomno );
}

void spreadroomdark( int x, int y, int roomno )
{
	if ( !inbounds( x, y ) )
		return;
		
	if ( !loc_statusp( x, y, LIT ) )
		return;
		
	if ( Level->site[ x ][ y ].roomnumber != roomno )
		return;
		
	lreset( x, y, LIT );
	lset( x, y, CHANGED );
	Level->site[ x ][ y ].showchar = SPACE;
	putspot( x, y, SPACE );
	
	spreadroomdark( x + 1, y, roomno );
	spreadroomdark( x, y + 1, roomno );
	spreadroomdark( x - 1, y, roomno );
	spreadroomdark( x, y - 1, roomno );
}

void display_pack( void )
{
	int i;

	if ( Player.packptr < 1 )
		print3( "Pack is empty." );
	else
	{
		menuclear();
		menuprint( "Items in Pack:\n" );
		for( i = 0; i < Player.packptr; i++ )
		{
			sprintf( Str1, "  %c: %s\n", i + 'A', itemid( Player.pack[ i ] ));
			menuprint( Str1 );
		}
		showmenu();
	}
}

void display_possessions( void )
{
	int i;

	for( i = 0; i < MAXITEMS; i++ ) 
		display_inventory_slot( i, FALSE );
}

void display_inventory_slot( int slotnum, int topline )
{
	WINDOW *W;
	char usechar = ')';

	if ( Player.possessions[ slotnum ] != NULL )
		if ( Player.possessions[ slotnum ]->used )
			usechar = '>';
	if ( topline )
		W = Msg3w;
	else
	{
		W = Showline[ slotnum ];
		hide_line( slotnum );
	}
	touchwin( W );
	wclear( W );
	
	switch( slotnum )
	{
		case O_UP_IN_AIR:
			wprintw( W, "-- Object 'up in air':", usechar );
			break;

		case O_READY_HAND:
			wprintw( W, "-- a%c ready hand: ", usechar );
			break;

		case O_WEAPON_HAND:
			wprintw( W, "-- b%c weapon hand: ", usechar );
			break;

		case O_LEFT_SHOULDER:
			wprintw( W, "-- c%c left shoulder: ", usechar );
			break;

		case O_RIGHT_SHOULDER:
			wprintw( W, "-- d%c right shoulder: ", usechar );
			break;

		case O_BELT1:
			wprintw( W, "-- e%c belt: ", usechar );
			break;

		case O_BELT2:
			wprintw( W, "-- f%c belt: ", usechar );
			break;

		case O_BELT3:
			wprintw( W, "-- g%c belt: ", usechar );
			break;

		case O_SHIELD:
			wprintw( W, "-- h%c shield: ", usechar );
			break;

		case O_ARMOR:
			wprintw( W, "-- i%c armor: ", usechar );
			break;

		case O_BOOTS:
			wprintw( W, "-- j%c boots: ", usechar );
			break;

		case O_CLOAK:
			wprintw( W, "-- k%c cloak: ", usechar );
			break;

		case O_RING1:
			wprintw( W, "-- l%c finger: ", usechar );
			break;

		case O_RING2:
			wprintw( W, "-- m%c finger: ", usechar );
			break;

		case O_RING3:
			wprintw( W, "-- n%c finger: ", usechar );
			break;

		case O_RING4:
			wprintw( W, "-- o%c finger: ", usechar );
			break;
	}
	if ( Player.possessions[ slotnum ] == NULL )
		wprintw( W, "(slot vacant)" );
	else
		wprintw( W, itemid( Player.possessions[ slotnum ] ));
	wrefresh( W );
}

int move_slot( int oldslot, int newslot, int maxslot )
{
	if (( newslot >= 0 ) && ( newslot < maxslot ))
	{
		wmove( Showline[ oldslot ], 0, 0 );
		waddstr( Showline[ oldslot ], "--" );
		wrefresh( Showline[ oldslot ] );
		wmove( Showline[ newslot ], 0, 0 );
		wstandout( Showline[ newslot ] );
		waddstr( Showline[ newslot ], ">>" );
		wstandend( Showline[ newslot ] );
		wrefresh( Showline[ newslot ] );
		return( newslot );
	}
	else
		return( oldslot );
}

void colour_on( void )
{
}

void colour_off( void )
{
	wattrset( Levelw, COL_WHITE >> 8 );
}

void display_option_slot( int slot )
{
	hide_line( slot );
	wclear( Showline[ slot ] );
	switch( slot )
	{
		case 1:
			wprintw( Showline[ slot ], "-- Option BELLICOSE [TF]: " );
			wprintw( Showline[ slot ], optionp( BELLICOSE ) ? "(now T) " : "(now F) " );
			break;
			
		case 2:
			wprintw( Showline[ slot ], "-- Option JUMPMOVE [TF]: " );
			wprintw( Showline[ slot ], optionp( JUMPMOVE ) ? "(now T) " : "(now F) " );
			break;
		
		case 3:
			wprintw( Showline[ slot ], "-- Option RUNSTOP [TF]: " );
			wprintw( Showline[ slot ], optionp( RUNSTOP ) ? "(now T) " : "(now F) " );
			break;
		
		case 4:
			wprintw( Showline[ slot ], "-- Option PICKUP [TF]: " );
			wprintw( Showline[ slot ], optionp( PICKUP ) ? "(now T) " : "(now F) " );
			break;
		
		case 5:
			wprintw( Showline[ slot ], "-- Option CONFIRM [TF]: " );
			wprintw( Showline[ slot ], optionp( CONFIRM ) ? "(now T) " : "(now F) " );
			break;
		
		case 6:
			wprintw( Showline[ slot ], "-- Option TOPINV [TF]: " );
			wprintw( Showline[ slot ], optionp( TOPINV ) ? "(now T) " : "(now F) " );
			break;
		
		case 7:
			wprintw( Showline[ slot ], "-- Option PACKADD [TF]: " );
			wprintw( Showline[ slot ], optionp( PACKADD ) ? "(now T) " : "(now F) " );
			break;
		
		case VERBOSITY_LEVEL:
			wprintw( Showline[ slot ],
					 "-- Option VERBOSITY [(T)erse,(M)edium,(V)erbose]: (now " );
			if ( Verbosity == VERBOSE )
				wprintw( Showline[ slot ], "Verbose)" );
			else if ( Verbosity == MEDIUM )
				wprintw( Showline[ slot ], "Medium)" );
			else
				wprintw( Showline[ slot ], "Terse)" );
			break;
		
		case SEARCH_DURATION:
			wprintw( Showline[ slot ], "-- Option SEARCHNUM [0>x>10]: (now %d)", Searchnum );
			break;
	}
	wrefresh( Showline[ slot ] );
}

void display_options( void )
{
	int i;

	menuclear();
	hide_line( 0 );
	for( i = 1; i <= NUMOPTIONS; i++ )
		display_option_slot( i );
}

/* nya ha ha ha ha haaaa.... */
void deathprint( void )
{
	mgetc();
	waddch( Msgw, 'D' );
	wrefresh( Msgw );
	mgetc();
	waddch( Msgw, 'e' );
	wrefresh( Msgw );
	mgetc();
	waddch( Msgw, 'a' );
	wrefresh( Msgw );
	mgetc();
	waddch( Msgw, 't' );
	wrefresh( Msgw );
	mgetc();
	waddch( Msgw, 'h' );
	wrefresh( Msgw );
	mgetc();
}
  
void clear_if_necessary( void )
{
	int x, y;

	getyx( Msg1w, y, x );
	if ( x != 0 )
	{
		wclear( Msg1w );
		wrefresh( Msg1w );
	}  

	getyx( Msg2w, y, x );
	if ( x != 0 )
	{
		wclear( Msg2w );
		wrefresh( Msg2w );
	}  

	getyx( Msg3w, y, x );
	if ( x != 0 )
	{
		wclear( Msg3w );
		wrefresh( Msg3w );
	}
}

int bufferpos = 0;
void buffercycle( char *s )
{
	strcpy( Stringbuffer[ bufferpos++ ], s );
	if ( bufferpos >= STRING_BUFFER_SIZE )
		bufferpos = 0;
}

int bufferappend( char *s )
{
	int pos = bufferpos - 1;

	if ( pos < 0 )
		pos = STRING_BUFFER_SIZE - 1;
	if ( strlen( Stringbuffer[ pos ] ) + strlen( s ) < 80 - 1 )
	{
		strcat( Stringbuffer[ pos ], s );
		return 1;
	}
	else
	return 0;
}

void bufferprint( void )
{
	int i = bufferpos - 1, c, finished = 0;

	clearmsg();
	wprintw( Msg1w, "^p for last message, ^n for next, anything else to quit." );
	wrefresh( Msg1w );
	do {
		if ( i >= STRING_BUFFER_SIZE )
			i = 0;
		if ( i < 0 )
			i = STRING_BUFFER_SIZE - 1;
		wclear( Msg2w );
		wprintw( Msg2w, Stringbuffer[ i ] );
		wrefresh( Msg2w );
		c = mgetc();
		if ( c == 16 )	/* ^p */
			i--;
		else if ( c == 14 )	/* ^n */
			i++;
		else
			finished = 1;
	} while ( !finished );
	clearmsg();
	omshowcursor( Player.x, Player.y );
}

void clear_screen( void )
{
	clear();
	touchwin( stdscr );
	refresh();
}
