/* omega copyright (C) by Laurence Raphael Brothers, 1987,1988,1989 */
/* site1.c */
/* 1st half of site functions and aux functions to them */

#include <ctype.h>
#include <unistd.h>
#include "curses.h"
#include "glob.h"
#include "ochars.h"

void set_correct_city_wall( int i, int j, plv Level );
void set_correct_hedge( int i, int j, plv Level );
void set_correct_pool( int i, int j, plv Level );

void bank_init( void );
void bank_add_account( bank_account *account, int alloc );
void bank_close_accounts( void );
void bank_break( WINDOW *w, int *jailed );
bank_account *bank_password( WINDOW *w, int *jailed );
void bank_deposit( WINDOW *w, bank_account *account );
void bank_withdraw( WINDOW *w, bank_account *account );
void bank_transaction( WINDOW *w, bank_account *account );
int bank_new_customer( WINDOW *w );
int bank_open_account( WINDOW *w );
long input_amount( WINDOW *w, int line, int col );
int input_password( WINDOW *w, int line, int col, char *buf, int allow_esc );

extern WINDOW *Bankw;

#define ABORT -1
#define NUM_NPC_ACCOUNTS 30
bank_account npc_account[ NUM_NPC_ACCOUNTS ] =
{
	{ FALSE,	3000,	"Duke",			NULL },
	{ FALSE,	30000,	"Duke Nukem",	NULL },
	{ FALSE,	300000,	"Dukes Ca$h",	NULL },
	{ FALSE,	100000, "SexyNoreen",	NULL },
	{ FALSE,	75000,	"ExSuccubus",	NULL },
	{ FALSE,	125000,	"Hooker Sue",	NULL },
	{ FALSE,	50000,	"JungleGirl",	NULL },
	{ FALSE,	75000,	"Satyrical",	NULL },
	{ FALSE,	50000,	"Fangless",		NULL },
	{ FALSE,	100000,	"Big Daddy",	NULL },
	{ FALSE,	50000,	"PraiseEros",	NULL },
	{ FALSE,	10000,	"Sonders",		NULL },
	{ FALSE,	300000,	"CSRFLP",		NULL },
	{ FALSE,	3000,	"Order",		NULL },
	{ FALSE,	3000,	"Diner",		NULL },
	{ FALSE,	50000,	"LC****",		NULL },
	{ FALSE,	5000,	"Eclipse",		NULL },
	{ FALSE,	25000,	"The Circle",	NULL },
	{ FALSE,	5000,	"College",		NULL },
	{ FALSE,	40000,	"FinanceAid",	NULL },
	{ FALSE,	10000,	"Librarian",	NULL },
	{ FALSE,	500000,	"RTG, Inc.",	NULL },
	{ FALSE,	10000,	"Alchemy",		NULL },
	{ FALSE,	200000, "LOD Mercs",	NULL },
	{ FALSE,	75000,	"KnightPawn",	NULL },
	{ FALSE,	10000,	"Julie",		NULL },
	{ FALSE,	40000,	"RampartGym",	NULL },
	{ FALSE,	3000,	"Arena",		NULL },
	{ FALSE,	2000,	"Slongo",		NULL },
	{ FALSE,	2000,	"Bedwyr",		NULL }
};

int account_num[ NUM_NPC_ACCOUNTS ];
bank_account cash_account = { FALSE, 500, "cash", NULL };
bank_account cheat_account = { TRUE, 0, "Bar", NULL };

void bank_init( void )
{
	int i;
	
	bank_add_account( &cash_account, FALSE );
	bank_add_account( &cheat_account, FALSE );
	shuffle( account_num, NUM_NPC_ACCOUNTS );
	for( i = 0; i < ( random_range( 6 ) + 3 ); i++ )
	{
		bank_add_account( &(npc_account[ account_num[ i ] ]), FALSE );
	}
}

void bank_add_account( bank_account *account, int alloc )
{
	bank_account *new_account;
	
	if (alloc)
	{
		new_account = (bank_account *)checkmalloc( sizeof( bank_account ) );
		new_account->player = account->player;
		new_account->password = salloc( account->password );
	}
	else
	{
		new_account = account;
	}
	
	if ( account->balance > 0 )
		new_account->balance = random_range( account->balance );
	else
		new_account->balance = 0;

	new_account->next_account = bank;
	bank = new_account;
}

int input_password( WINDOW *w, int line, int col, char *buf, int allow_esc )
{
	int pwlen = 0;
	char *cp = buf;
	
	for (;;)
	{
		*cp = (char)mvwgetch( w, line, col + pwlen );
		if ( *cp == '\r' || *cp == '\n' )
		{
			*cp = '\0';
			break;
		}
		else if ( *cp == KEY_ESCAPE && allow_esc )
		{
			*buf = KEY_ESCAPE;
			return 1;
		}
		else if ( *cp == KEY_LEFT || *cp == KEY_DELETE )
		{
			if ( pwlen > 0 )
			{
				cp--;
				pwlen--;
				mvwaddch( w, line, col + pwlen, ' ' );
			}
		}
		else if ( isprint( *cp ) && pwlen < 10 )
		{
			mvwaddch( w, line, col + pwlen, '*' );
			pwlen++;
			cp++;
		}
		
		wrefresh( w );
	}
	
	return pwlen;
}

long input_amount( WINDOW *w, int line, int col )
{
	char c;
	int amountlen = 0;
	long sign = 1;
	long amount = 0;
	
	for (;;)
	{
		c = (char)mvwgetch( w, line, col + amountlen );
		if ( c == '\r' || c == '\n' )
		{
			return sign * amount;
			break;
		}
		else if ( c == KEY_LEFT || c == KEY_DELETE )
		{
			if ( amountlen > 0 )
			{
				amount /= 10;
				amountlen--;
				mvwaddch( w, line, col + amountlen, ' ' );
			}
			
			if ( amountlen == 0 )
				sign = 1;
		}
		else if ( amountlen == 0 && ( c == '-' || c == '+' ))
		{
			mvwaddch( w, line, col + amountlen, c );
			amountlen++;
			if ( c == '-' )
				sign = -1;
		}
		else if ( isdigit( c ) && amountlen < 8 )
		{
			mvwaddch( w, line, col + amountlen, c );
			amountlen++;
			amount = ( amount * 10 ) + ( c - '0' );
		}
		
		wrefresh( w );
	}
}

int bank_new_customer( WINDOW *w )
{
	int response;
	
	mvwaddstr( w, 4, 2, "ARE YOU A NEW CUSTOMER? [YN]: " );
	wrefresh( w );
	
	for(;;)
	{
		response = toupper( wgetch( w ));
		if ( response == 'Y' || response == 'N' || response == KEY_ESCAPE )
		{
			mvwaddch( w, 4, 32, response );
			wrefresh( w );
			break;
		}
	}
	
	if ( response == KEY_ESCAPE )
		return ABORT;
	else if ( response == 'Y' )
		return TRUE;
	else
		return FALSE;
}

int bank_open_account( WINDOW *w )
{
	int pwlen, response;
	int got_password;
	bank_account *account;
	bank_account new_account;
	char pw_buf1[ 16 ];
	char pw_buf2[ 16 ];
	
	do {
	
		mvwaddstr( w, 5, 2, "OPEN AN ACCOUNT? [YN]: " );
		wrefresh( w );
		
		for(;;)
		{
			response = toupper( wgetch( w ));
			if ( response == 'Y' || response == 'N' )
			{
				mvwaddch( w, 5, 25, response );
				wrefresh( w );
				break;
			}
		}
		
		if ( response == 'N' )
		{
			mvwaddstr( w, 4, 2, "                                     " );
			mvwaddstr( w, 5, 2, "                                     " );
			mvwaddstr( w, 7, 9, "PLEASE REMEMBER US FOR" );
			mvwaddstr( w, 8, 11, "YOUR BANKING NEEDS" );
			wrefresh( w );
			sleep( 3 );
			return FALSE;
		}
		
		mvwaddstr( w, 7, 8, "[ OPENING NEW ACCOUNT ]" );
		mvwaddstr( w, 9, 2, "PLEASE ENTER A PASSWORD: " );
		wrefresh( w );
			
		for (;;)
		{
			pwlen = input_password( w, 9, 27, pw_buf1, FALSE );
			if ( pwlen )
				break;
	
			mvwaddstr( w, 11, 2, "NULL PASSWORDS NOT ALLOWED" );
			mvwaddstr( w, 9, 27, "            " );
			wrefresh( w );
		}
		
		mvwaddstr( w, 10, 2, "PLEASE CONFIRM PASSWORD: " );
		mvwaddstr( w, 11, 2, "                          " );
		wrefresh( w );
		
		for (;;)
		{
			pwlen = input_password( w, 10, 27, pw_buf2, FALSE );
			if ( pwlen )
				break;
	
			mvwaddstr( w, 12, 2, "NULL PASSWORDS NOT ALLOWED" );
			mvwaddstr( w, 10, 27, "            " );
			wrefresh( w );
			wrefresh( w );
		}
		
		if ( 0 == strcmp( pw_buf1, pw_buf2 ))
		{
			/* search for preexisting account with that password */
			for( account = bank; account; account = account->next_account )
			{
				if ( 0 == strcmp( pw_buf1, account->password ))
					break;
			}
			
			if ( account )
			{
				mvwaddstr( w, 12, 2, "PASSWORD NOT VALID; RESTARTING.      " );
				got_password = FALSE;
			}
			else
			{
				new_account.player = TRUE;
				new_account.balance = 0;
				new_account.password = pw_buf1;
				bank_add_account( &new_account, TRUE );
				mvwaddstr( w, 12, 2, "PASSWORD VALIDATED; ACCOUNT SAVED.   " );
				got_password = TRUE;
			}
		}
		else
		{
			mvwaddstr( w, 12, 2, "PASSWORDS DIDN'T MATCH; RESTARTING.  " );
			got_password = FALSE;
		}
		wrefresh( w );
		sleep( 3 );
		
		mvwaddstr( w, 5, 2, "                                     " );
		mvwaddstr( w, 7, 2, "                                     " );
		mvwaddstr( w, 9, 2, "                                     " );
		mvwaddstr( w, 10, 2, "                                     " );
		mvwaddstr( w, 12, 2, "                                     " );
	
	} while ( !got_password );
	
	return TRUE;
}

void bank_close_accounts( void )
{
	bank_account *account, *next;
	
	for( account = bank; account; account = next )
	{
		next = account->next_account;

        /* don't free npc accounts (they're static) */
        if ((account >= npc_account) && ((account - npc_account) < NUM_NPC_ACCOUNTS))
          continue;

        /* don't free other static accounts */
        if (account == &cash_account || account == &cheat_account)
          continue;

		free( account->password );
		free( account );
	}
	
	bank = NULL;
}

char *bb_garbage = "^@^@^@^@^@00AD1203BC0F0000FFFFFFFFF";
void bank_break( WINDOW *w, int *jailed )
{
	int response;
	long total_balance;
	char *cp;
	bank_account *account;
	
	mvwaddstr( w, 6, 2, "  ALERT! ALERT! INVALID PASSWORD!    " );
	mvwaddstr( w, 7, 2, "   THE POLICE ARE BEING SUMMONED!    " );
	mvwaddstr( w, 8, 2, "            PLEASE WAIT              " );
	mvwaddstr( w, 9, 2, "     FOR THE POLICE TO ARRIVE.       " );
	mvwaddstr( w, 11, 2, " --- HIT SPACE BAR TO CONTINUE ---   " );
	wrefresh( w );
	
	response = wgetch( w );
	if ( response == ' ' )
	{
		Player.alignment += 5;
		print1( "Ah ha! Trying to rob the bank, eh?" );
		print2( "Take him away, boys!" );
		morewait();
		send_to_jail();
		*jailed = TRUE;
	}
	else
	{
		Player.alignment -= 5;
		mvwaddstr( w, 2, 2, "                                     " );
		mvwaddstr( w, 4, 2, "                                     " );
		mvwaddstr( w, 6, 2, "                                     " );
		mvwaddstr( w, 7, 2, "                                     " );
		mvwaddstr( w, 8, 2, "                                     " );
		mvwaddstr( w, 9, 2, "                                     " );
		mvwaddstr( w, 11, 2, "                                     " );
		wrefresh( w );
		
		wmove( w, 2, 2 );
		for( cp = bb_garbage; *cp; cp++ )
		{
			waddch( w, *cp );
			wrefresh( w );
			usleep( 100000 );
		}
		
		mvwaddstr( w, 3, 2, "ERR _GET_SPACE(): ILLEGAL CHARACTER  " );
		wrefresh( w );
		sleep( 4 );
		mvwaddstr( w, 4, 2, "ABORTING _POLICE_ALERT().            " );
		mvwaddstr( w, 5, 2, "ATTEMPTING REBOOT...                 " );
		wrefresh( w );
		sleep( 4 );
		mvwaddstr( w, 6, 2, "WARN: _COUNT_CASH(): ILLEGAL SHMOP   " );
		mvwaddstr( w, 7, 2, "WARN: COMMAND BUFFER *NOT CLEARED*   " );
		wrefresh( w );
		sleep( 4 );
		mvwaddstr( w, 8, 2, "REBOOT DONE. EXECUTION CONTINUING.   " );
		mvwaddstr( w, 9, 2, "WITHDRAWING 4294967297 AU            " );
		mvwaddstr( w, 10, 2, "WARN: _WITHDRAW(): INTEGER OVERFLOW  " );
		wrefresh( w );
		sleep( 4 );
		mvwaddstr( w, 11, 2, "YO MAMA. CORE DUMPED.                " );
		mvwaddstr( w, 13, 2, "CALL MICROSOFT EMBEDDED PRODUCTS     " );
		mvwaddstr( w, 14, 2, "TECHNICAL SUPPORT FOR ASSISTANCE     " );
		wrefresh( w );
		sleep( 6 );
		
		total_balance = 0;
		for( account = bank; account; account = account->next_account )
		{
			if ( account->player )
				total_balance += account->balance;
		}
		
		clearmsg();
		print1( "The cash machine begins to spew gold pieces!" );
		if ( total_balance > 0 )
			print2( "You pick up your entire balance and then some!" );
		else
			print2( "You eagerly collect the coins in your purse!" );
			
		Player.cash += total_balance + 1000 + random_range( 3000 );
		bank_close_accounts();
		setgamestatus( BANK_BROKEN );
		dataprint();
	}
}

bank_account *bank_password( WINDOW *w, int *jailed )
{
	bank_account *account;
	char pw_buf[ 16 ];
	
	mvwaddstr( w, 4, 2, "ENTER YOUR PASSWORD:                 " );
	wrefresh( w );
	
	(void)input_password( w, 4, 23, pw_buf, TRUE );
	
	if ( pw_buf[ 0 ] == KEY_ESCAPE )
		return NULL;
	
	for( account = bank; account; account = account->next_account )
	{
		if ( 0 == strcmp( account->password, pw_buf ))
			break;
	}
	
	if ( account )
	{
		mvwaddstr( w, 4, 2, "PASSWORD ACCEPTED. WORKING           " );
		wrefresh( w );
		
		if ( !account->player )
		{
			Player.alignment -= 5;
		}
		
		if ( account == &cheat_account )
		{
			setgamestatus( CHEATED );
			mprint( "Wizard mode set." );
		}
	}
	else
	{
		bank_break( w, jailed );
	}
	
	return account;
}

void bank_deposit( WINDOW *w, bank_account *account )
{
	long amount;
	
	mvwaddstr( w, 12, 2, "AMOUNT:                              " );
	wrefresh( w );
	
	amount = input_amount( w, 12, 10 );
	
	if ( account == &cheat_account )
	{
		mvwaddstr( w, 14, 2, "TRANSACTION ACCOMPLISHED             " );
		Player.cash -= amount;
	}
	else if ( amount < 0 )
	{
		mvwaddstr( w, 14, 2, "UNAUTHORIZED WITHDRAWAL ATTEMPT!     " );
		mvwaddstr( w, 15, 2, "A FINE OF 1000 AU HAS BEEN LEVIED    " );
		mvwaddstr( w, 16, 2, "AGAINST YOUR ACCOUNT.                " );
		account->balance -= 1000;
	}
	else if ( amount == 0 )
	{
		mvwaddstr( w, 14, 2, "TRANSACTION ABORTED                  " );
	}
	else if ( amount > Player.cash )
	{
		mvwaddstr( w, 14, 2, "                                     " );
		mvwprintw( w, 14, 2, "%d AU DEPOSITED", Player.cash );
		account->balance += Player.cash;
		Player.cash = 0;
	}
	else
	{
		mvwaddstr( w, 14, 2, "TRANSACTION ACCOMPLISHED             " );
		account->balance += amount;
		Player.cash -= amount;
	}
	
	wrefresh( w );
}

void bank_withdraw( WINDOW *w, bank_account *account )
{
	long amount;
	
	mvwaddstr( w, 12, 2, "AMOUNT:                              " );
	wrefresh( w );
	
	amount = input_amount( w, 12, 10 );
	
	if ( account == &cheat_account )
	{
		mvwaddstr( w, 14, 2, "TRANSACTION ACCOMPLISHED             " );
		Player.cash += amount;
	}
	else if ( amount == 0 )
	{
		mvwaddstr( w, 14, 2, "TRANSACTION ABORTED                  " );
	}
	else if ( amount < 0 )
	{
		if ( abs( amount ) > Player.cash )
		{
			mvwaddstr( w, 14, 2, "                                     " );
			mvwprintw( w, 14, 2, "%d AU DEPOSITED", Player.cash );
			account->balance += Player.cash;
			Player.cash = 0;
		}
		else
		{
			mvwaddstr( w, 14, 2, "TRANSACTION ACCOMPLISHED             " );
			account->balance -= amount;
			Player.cash += amount;
		}
	}
	else if ( amount > account->balance )
	{
		mvwaddstr( w, 14, 2, "ACCOUNT OVERDRAFT! A FINE OF         " );
		mvwaddstr( w, 15, 2, "1000 AU HAS BEEN LEVIED AGAINST      " );
		mvwaddstr( w, 16, 2, "YOUR ACCOUNT.                        " );
		if ( account->balance > 0 )
		{
			Player.cash += account->balance;
			account->balance = -1000;
		}
		else
			account->balance -= 1000;
	}
	else
	{
		mvwaddstr( w, 14, 2, "TRANSACTION ACCOMPLISHED             " );
		account->balance -= amount;
		Player.cash += amount;
	}
	
	wrefresh( w );
}

void bank_transaction( WINDOW *w, bank_account *account )
{
	short response;
	short exit = FALSE;

	do {
	
		if ( account == &cheat_account )
			mvwprintw( w, 6, 2, "CURRENT BALANCE: INFINITY", account->balance );
		else
		{
			mvwaddstr( w, 6, 2, "                                     " );
			mvwprintw( w, 6, 2, "CURRENT BALANCE: %d AU", account->balance );
		}
		mvwaddstr( w, 8, 2, " D: DEPOSIT" );
		mvwaddstr( w, 9, 2, " W: WITHDRAW" );
		mvwaddstr( w, 10, 2, " E: EXIT" );
		mvwaddstr( w, 12, 2, "ENTER COMMAND:  " );
		wrefresh( w );
		
		mvwaddstr( w, 14, 2, "                                     " );
		mvwaddstr( w, 15, 2, "                                     " );
		mvwaddstr( w, 16, 2, "                                     " );
		
		wmove( w, 12, 17 );
		response = toupper( wgetch( w ));
		switch( response )
		{
			case 'D':
				mvwaddstr( w, 9, 2, "                                     " );
				mvwaddstr( w, 10, 2, "                                     " );
				bank_deposit( w, account );
				break;
				
			case 'W':
				mvwaddstr( w, 8, 2, "                                     " );
				mvwaddstr( w, 10, 2, "                                     " );
				bank_withdraw( w, account );
				break;
				
			case 'E':
			case KEY_ESCAPE:
				exit = TRUE;
				break;
				
			default:
				beep();
				mvwaddstr( w, 14, 2, "INVALID ENTRY" );
				wrefresh( w );
				break;
		}
		
		dataprint();
	
	} while( !exit );
}

/* the bank; can be broken into (!) */
void l_bank( void )
{
	int new_cust;
	int jailed = FALSE;
	int has_account = TRUE;
	bank_account *account;
	
	wattrset( Bankw, BANKCHAR_OFFSET );
	werase( Bankw );
	wattrset( Bankw, 0 );
	box( Bankw, 0, 0 );
	wattrset( Bankw, BANKCHAR_OFFSET );
	mvwaddstr( Bankw, 2, 3, "WELCOME TO THE FIRST BANK OF OMEGA" );

	if ( gamestatusp( BANK_BROKEN ))
	{
		mvwaddstr( Bankw, 9, 8, "AUTOTELLER OUT OF ORDER" );
		wrefresh( Bankw );
		print1( "You see a damaged autoteller." );
		morewait();
	}
	else
	{
		print1( "The proximity sensor activates the autoteller as you approach." );
		new_cust = bank_new_customer( Bankw );
		
		if ( new_cust != ABORT )
		{
			if ( new_cust == TRUE )
			{
				has_account = bank_open_account( Bankw );
			}
			
			if ( has_account )
			{
				account = bank_password( Bankw, &jailed );
				if ( account )
				{
					bank_transaction( Bankw, account );
				}
	
				if ( !jailed && !gamestatusp( BANK_BROKEN ))
				{
					mvwaddstr( Bankw, 4, 2, "       THANK YOU FOR CHOOSING        " );
					mvwaddstr( Bankw, 5, 2, "      THE FIRST BANK OF OMEGA!       " );
					mvwaddstr( Bankw, 6, 2, "                                     " );
					mvwaddstr( Bankw, 8, 2, "                                     " );
					mvwaddstr( Bankw, 9, 2, "                                     " );
					mvwaddstr( Bankw, 10, 2, "                                     " );
					mvwaddstr( Bankw, 12, 2, "                                     " );
					wrefresh( Bankw );
					sleep( 3 );
				}
			}
		}
	}
	
	if ( !jailed && !gamestatusp( BANK_BROKEN ))
		clearmsg();
	
	xredraw();
}

void l_armorer( void )
{
	int done = FALSE;
	char action;
	
	if ( hour() == 12 ) 
		print3( "Unfortunately, this is Julie's lunch hour -- try again later." );
	else if ( nighttime() ) 
		print3( "It seems that Julie keeps regular business hours." );
	else
	{
		while ( !done )
		{
			clearmsg();
			print1( "Julie's: Buy Armor, Weapons, or Leave [a,w,ESCAPE] " );
			action = mgetc();
			if ( action == ESCAPE ) 
				done = TRUE;
			else if ( action == 'a' ) 
				buyfromstock( ARMORID, 10 );
			else if ( action == 'w' ) 
				buyfromstock( WEAPONID, 23 );
		}
	}
	xredraw();
}

void buyfromstock( int base, int numitems )
{
	int i;
	char item;
	pob newitem;

	print2( "Purchase which item? [ESCAPE to quit] " );
	menuclear();
	for( i = 0; i < numitems; i++ )
	{
		strcpy( Str4, " :" );
		Str4[ 0 ] = i + 'a';
		strcat( Str4, Objects[ base + i ].objstr );
		menuprint( Str4 );
		menuprint( "\n" );
	}
	showmenu();
	item = ' ';

	while (( item != ESCAPE ) && (( item < 'a' ) || ( item >= 'a' + numitems )))
		item = mgetc();

	if ( item != ESCAPE )
	{
		i = item - 'a';
		newitem = ( (pob)checkmalloc( sizeof( objtype )));
		*newitem = Objects[ base + i ];
		newitem->known = 2;
		clearmsg();
		print1( "I can let you have it for " );
		mlongprint( 2 * true_item_value( newitem ));
		nprint1( "Au. Buy it? [yn] " );
		if ( ynq1() == 'y' )
		{
			if ( Player.cash < 2 * true_item_value( newitem ))
			{
				print2( "Why not try again some time you have the cash?" );
				free( (char *)newitem );
			}
			else
			{
				Player.cash -= 2 * true_item_value( newitem );
				dataprint();
				gain_item( newitem );
			}
		}
		else
			free( (char *)newitem );
	}
}

void l_club( void )
{
#define hinthour club_hinthour
	char response;

	print1( "Rampart Explorers' Club." );
	if ( !gamestatusp( CLUB_MEMBER ))
	{
		if ( Player.level < 2 )
			print3( "Only reknowned adventurers need apply." );
		else
		{
			print2( "Dues are 100Au. Pay it? [yn] " );
			if ( ynq2() == 'y' )
			{
				if ( Player.cash < 100 )
					print3( "Beat it, or we'll blackball you!" );
				else
				{
					print1( "Welcome to the club! You are taught the spell of Return." );
					print2( "When cast on the first level of a dungeon it" );
					morewait();
					clearmsg();
					print1( "will transport you down to the lowest level" );
					print2( "you have explored, and vice versa." );
					Spells[ S_RETURN ].known = TRUE;
					Player.cash -= 100;
					setgamestatus( CLUB_MEMBER );
				}
			}
			else
				print2( "OK, but you're missing out on our benefits...." );
		}
	}
	else
	{
		print2( "Shop at the club store or listen for rumors [sl] " );
		do response = (char)mcigetc(); 
		while (( response != 's' ) && ( response != 'l' ) && ( response != ESCAPE ));
		if ( response == 'l' )
		{
			if ( hinthour == hour() )
				print2( "You don't hear anything useful." );
			else
			{
				print1( "You overhear a conversation...." );
				hint();
				hinthour = hour();
			}
		}
		else if ( response == 's' )
		{
			buyfromstock( THINGID + 7, 2 );
			xredraw();
		}
		else if ( response == ESCAPE )
			print2( "Be seeing you, old chap!" );
	}
}
#undef hinthour

void l_gym( void )
{
	int done = TRUE;

	clearmsg();
	do {
		print1( "The Rampart Gymnasium" );
		if (( Gymcredit > 0 ) || ( Player.rank[ ARENA ] ))
		{
			nprint1( "-- Credit: " );
			mlongprint( Gymcredit );
			nprint1( "Au." );
		}
		done = FALSE;
		menuclear();
		menuprint( "Train for 2000 Au. Choose:\n" );
		menuprint( "\na: work out in the weight room" );
		menuprint( "\nb: use our gymnastics equipment" );
		menuprint( "\nc: take our new anaerobics course" );
		menuprint( "\nd: enroll in dance lessons." );
		menuprint( "\nESCAPE: Leave this place." );
		showmenu();
		switch( mgetc() )
		{
			case 'a': 
				gymtrain( &( Player.maxstr ), &( Player.str ));
				break;

			case 'b': 
				gymtrain( &( Player.maxdex ), &( Player.dex ));
				break;

			case 'c': 
				gymtrain( &( Player.maxcon ), &( Player.con ));
				break;

			case 'd':
				gymtrain( &( Player.maxagi ), &( Player.agi ));
				break;

			case ESCAPE:
				clearmsg();
				print1( "Well, it's your body you're depriving!" ); 
				done = TRUE;
				break;
		}
	} while ( !done );
	xredraw();
	calc_melee();
}

void l_healer( void )
{
	print1( "Rampart Healers. Member RMA." );
	morewait();
	clearmsg();
	print1( "a: Heal injuries (50 crowns)" );
	print2( "b: Cure disease (250 crowns)" );
	print3( "ESCAPE: Leave these antiseptic alcoves." );
	switch( (char)mcigetc() )
	{
		case 'a': healforpay(); break;
		case 'b': cureforpay(); break;
		default: print3( "OK, but suppose you have Acute Satyriasis?" ); break;
	}	      
}

void statue_random( int x, int y )
{
	pob item;
	int i, j;
	
	switch( random_range( difficulty() + 3 ) - 1 )
	{
		default:
			l_statue_wake();
			break;
		
		case 0: 
			print1( "The statue crumbles with a clatter of gravel." );
			Level->site[ x ][ y ].locchar = MY_RUBBLE;
			Level->site[ x ][ y ].p_locf = L_RUBBLE;
			plotspot( x, y, TRUE );
			lset( x, y, CHANGED );
			break;

		case 1: 
			print1( "The statue stoutly resists your attack." );
			break;

		case 2: 
			print1( "The statue crumbles with a clatter of gravel." );
			Level->site[ x ][ y ].locchar = MY_RUBBLE;
			Level->site[ x ][ y ].p_locf = L_RUBBLE;
			plotspot( x, y, TRUE );
			lset( x, y, CHANGED );
			make_site_treasure( x, y, difficulty() );
			break;

		case 3:
			print1( "The statue hits you back!" );
			p_damage( random_range( difficulty() * 5 ), UNSTOPPABLE, "a statue" );
			break;

		case 4:
			print1( "The statue looks slightly pained. It speaks:" );
			morewait();
			clearmsg();
			hint();
			break;

/*
     if ((Current_Environment == Current_Dungeon &&
         Level->depth < MaxDungeonLevels) ||
         (Current_Environment == E_CITY)) {
*/
		case 5:
			if ((( Current_Environment == Current_Dungeon ) ||
				( Current_Environment == E_CITY )) &&
				!( Current_Environment == E_SEWERS && Level->depth == SEWERLEVELS ) &&
				!( Current_Environment == E_CAVES && Level->depth == CAVELEVELS ) &&
				!( Current_Environment == E_CASTLE && Level->depth == CASTLELEVELS ) &&
				!( Current_Environment == E_ASTRAL && Level->depth == ASTRALLEVELS ) &&
				!( Current_Environment == E_VOLCANO && Level->depth == VOLCANOLEVELS ))
			{
				print1( "You hear the whirr of some mechanism." );
				print2( "The statue glides smoothly into the floor!" );
				Level->site[ x ][ y ].locchar = MY_STAIRS_DOWN;
				Level->site[ x ][ y ].p_locf = L_NO_OP;
				lset( x, y, CHANGED );
			}
			break;

		case 6:
			print1( "The statue was covered with contact cement!" );
			print2( "You can't move...." );
			Player.status[ IMMOBILE ] += random_range( 6 ) + 2;
			break;

		case 7:
			print1( "A strange radiation emanates from the statue!" );
			dispel( -1 );
			break;

		case 8: /* I think this is particularly evil. Heh heh. */
			if ( Player.possessions[ O_WEAPON_HAND ] != NULL )
			{
				print1( "Your weapon sinks deeply into the statue and is sucked away!" );
				item = Player.possessions[ O_WEAPON_HAND ];
				conform_lost_object( Player.possessions[ O_WEAPON_HAND ] );
				item->blessing = -1 - abs( item->blessing );
				drop_at( x, y, item );
			}
			break;

		case 9:
			print1( "The statue extends an arm. Beams of light illuminate the level!" );
			for( i = 0; i < WIDTH; i++ )
				for( j = 0; j < LENGTH; j++ )
				{
					lset( i, j, SEEN );
					if ( loc_statusp( i, j, SECRET ))
					{
						lreset( i, j, SECRET );
						lset( i, j, CHANGED );
					}
					if ( Level->site[ i ][ j ].locchar == WALL ||
						 ( Level->site[ i ][ j ].locchar >= WALL_FIRST &&
						   Level->site[ i ][ j ].locchar <= WALL_LAST ))
					{
						set_correct_city_wall( i, j, Level );
					}
					else if ( Level->site[ i ][ j ].locchar == HEDGE ||
							  ( Level->site[ i ][ j ].locchar >= HEDGE_FIRST &&
								Level->site[ i ][ j ].locchar <= HEDGE_LAST ))
					{
						set_correct_hedge( i, j, Level );
					}
					else if ( Level->site[ i ][ j ].locchar == WATER ||
							  ( Level->site[ i ][ j ].locchar >= POOL_FIRST &&
								Level->site[ i ][ j ].locchar <= POOL_LAST ))
					{
						set_correct_pool( i, j, Level );
					}
				}
			show_screen();
			break;
	}
}

void l_statue_wake( void )
{
	int i;
	int x = Player.x, y = Player.y;

	for( i = 0; i < 9; i++ )
		wake_statue( x + Dirs[ 0 ][ i ], y + Dirs[ 1 ][ i ], TRUE );
}

void wake_statue( int x, int y, int first )
{
	int i;
	pml tml;

	if ( Level->site[ x ][ y ].locchar == STATUE ||
		 Level->site[ x ][ y ].locchar == MY_STATUE )
	{
		if ( !first )
			mprint( "Another statue awakens!" );
		else
			mprint( "A statue springs to life!" );
		Level->site[ x ][ y ].locchar = GROUND;
		lset( x, y, CHANGED );
		tml = ( (pml)checkmalloc( sizeof( mltype )));
		tml->m = ( Level->site[ x ][ y ].creature = m_create( x, y, 0, difficulty() + 1 ));
		m_status_set( Level->site[ x ][ y ].creature, HOSTILE );
		tml->next = Level->mlist;
		Level->mlist = tml;
		for( i = 0; i < 8; i++ )
			wake_statue( x + Dirs[ 0 ][ i ], y + Dirs[ 1 ][ i ], FALSE );
	}
}

void l_casino( void )
{
	int i, done = FALSE, a, b, c, match;
	char response;

	print1( "Rampart Mithril Nugget Casino." );
	if ( random_range( 10 ) == 1 )
		print2( "Casino closed due to Grand Jury investigation." );
	else
	{
		while ( !done )
		{
			morewait();
			clearmsg();
			print1( "a: Drop 100Au in the slots." );
			print2( "b: Risk 1000Au  at roulette." );
			print3( "ESCAPE: Leave this green baize hall." );
			response = (char)mcigetc();
			if ( response == 'a' )
			{
				if ( Player.cash < 100 )
					print3( "No credit, jerk." );
				else
				{
					Player.cash -= 100;
					dataprint();
					for( i = 0; i < 20; i++ )
					{
						if ( i == 19 )
							sleep( 1 );
						else
							usleep( 250000 );

						a = random_range( 10 );
						b = random_range( 10 );
						c = random_range( 10 );
						clearmsg1();
						mprint( slotstr( a ));
						mprint( slotstr( b ));
						mprint( slotstr( c ));
					}
					
					if ( winnings > 0 )
						do {
							a = random_range( 10 );
							b = random_range( 10 );
							c = random_range( 10 );
						} while (( a == b ) || ( a == c ) || ( b == c ));
					else
					{
						a = random_range( 10 );
						b = random_range( 10 );
						c = random_range( 10 );
					}
					clearmsg();
					mprint( slotstr( a ));
					mprint( slotstr( b ));
					mprint( slotstr( c ));
					if (( a == b ) && ( a == c ))
					{
						print3( "Jackpot Winner!" );
						winnings += ( a + 2 ) * ( b + 2 ) * ( c + 2 ) * 5;
						Player.cash += ( a + 2 ) * ( b + 2 ) * ( c + 2 ) * 5;
						dataprint();
					}
					else if ( a == b )
					{
						print3( "Winner!" );
						Player.cash += ( a + 2 ) * ( b + 2 ) * 5;
						dataprint();
						winnings += ( a + 2 ) * ( b + 2 ) * 5;
					}
					else if ( a == c )
					{
						print3( "Winner!" );
						Player.cash += ( a + 2 ) * ( c + 2 ) * 5;
						dataprint();
						winnings += ( a + 2 ) * ( c + 2 ) * 5;
					}
					else if ( c == b )
					{
						print3( "Winner!" );
						Player.cash += ( c + 2 ) * ( b + 2 ) * 5;
						dataprint();
						winnings += ( c + 2 ) * ( b + 2 ) * 5;
					}
					else
					{
						print3( "Loser!" );
						winnings -= 100;
					}
				}
			}
			else if ( response == 'b' )
			{
				if ( Player.cash < 1000 )
					mprint( "No credit, jerk." );
				else
				{
					Player.cash -= 1000;
					dataprint();
					print1( "Red or Black? [rb]" );
					do response = (char)mcigetc();
					while (( response != 'r' ) && ( response != 'b' ));
					match = ( response == 'r' ? 0 : 1 );
					for( i = 0; i < 20; i++ )
					{
						if ( i == 19 )
							sleep( 1 );
						else
							usleep( 250000 );
						a = random_range( 37 );
						b = a % 2;
						if ( a == 0 )
							print1( " 0 " );
						else if ( a == 1 )
							print1( " 0 - 0 " );
						else
						{
							print1(( b == 0) ? "Red " : "Black " );
							mnumprint( a - 1 );
						}
					}
					if ( winnings > 0 )
						do {
							a = random_range( 37 );
							b = a % 2;
						} while ( b == match );
					else {
						a = random_range( 37 );
						b = a % 2;
					}
					if ( a == 0 )
						print1( " 0 " );
					else if ( a == 1 )
						print1( " 0 - 0 " );
					else
					{
						print1(( b == 0 ) ? "Red " : "Black " );
						mnumprint( a - 1 );
					}
					if (( a > 1 ) && ( b == match ))
					{
						print3( " Winner!" );
						winnings += 1000;
						Player.cash += 2000;
						dataprint();
					}
					else
					{
						print3( " Loser!" );
						winnings -= 1000;
						dataprint();
					}
				}
			}
			else if ( response == ESCAPE )
				done = TRUE;
		}
	}
}

void l_commandant( void )
{
	int num;
	pob food;

	print1( "Commandant Sonder's Rampart-fried Lyzzard partes. Open 24 hrs." );
	print2( "Buy a bucket! Only 5 Au. Make a purchase? [yn] " );
	if ( ynq2() == 'y' )
	{
		clearmsg();
		print1( "How many? " );
		num = (int)parsenum();
		if ( num < 1 )
			print3( "Cute. Real cute." );
		else if ( num * 5 > Player.cash )
			print3( "No handouts here, mac!" );
		else
		{
			Player.cash -= num * 5;
			food = ( (pob)checkmalloc( sizeof( objtype )));
			*food = Objects[ FOODID + 0 ]; /* food ration */
			food->number = num;
			if ( num == 1 )
				print2( "There you go, mac! One Lyzzard Bucket, coming up." );
			else
				print2( "A passel of Lyzzard Buckets, for your pleasure." );
			morewait();
			gain_item( food );
		}
	}
	else
		print2( "Don't blame the Commandant if you starve!" );
}

void l_diner( void )
{
	print1( "The Rampart Diner. All you can eat, 25Au." );
	print2( "Place an order? [yn] " );
	if ( ynq2() == 'y' )
	{
		if ( Player.cash < 25 )
			mprint( "TANSTAAFL! Now git!" );
		else
		{
			Player.cash -= 25;
			dataprint();
			Player.food = 44;
			foodcheck();
		}
	}
}

void l_crap( void )
{
	print1( "Les Crapeuleaux. (****) " );
	if (( hour() < 17 ) || ( hour() > 23 ))
		print2 ( "So sorry, we are closed 'til the morrow..." );
	else
	{
		print2( "May I take your order? [yn] " );
		if ( ynq2() == 'y' )
		{
			if ( Player.cash < 1000 )
				print2( "So sorry, you have not the funds for dinner." );
			else
			{
				print2( "Hope you enjoyed your tres expensive meal, m'sieur..." );
				Player.cash -= 1000;
				dataprint();
				Player.food += 8;
				foodcheck();
			}
		}
	}
}

void l_tavern( void )
{
#define hinthour tavern_hinthour
	char response;

	print1( "The Centaur and Nymph -- J. Riley, prop." );
	if ( nighttime() )
	{
		menuclear();
		menuprint( "Riley says: Whataya have?\n\n" );
		menuprint( "a: Pint of Riley's ultra-dark 1Au\n" );
		menuprint( "b: Shot of Tullimore Dew 10Au\n" );
		menuprint( "c: Round for the House. 100Au\n" );
		menuprint( "d: Bed and Breakfast. 25Au\n" );
		menuprint( "ESCAPE: Leave this comfortable haven.\n" );
		showmenu();
		do response = (char)mcigetc();
		while (( response != 'a' ) && ( response != 'b' ) && ( response != 'c' ) &&
			   ( response != 'd' ) && ( response != ESCAPE ));
		switch ( response )
		{
			case 'a':
				if ( Player.cash < 1 )
					print2( "Aw hell, have one on me." );
				else
				{
					hinthour = hour();
					Player.cash -= 1;
					dataprint();
					if ( hinthour != hour() )
					{
						if ( random_range( 3 ))
						{
							print1( "You overhear a rumor..." );
							hint();
						}
						else
							print1( "You don't hear much of interest." );
						hinthour = hour();
					}
					else
						print1( "You just hear the same conversations again." );
				}
				break;

			case 'b':
				if ( Player.cash < 10 )
					print2( "I don't serve the Dew on no tab, buddy!" );
				else
				{
					Player.cash -= 10;
					print1( "Ahhhhh...." );
					if ( Player.status[ POISONED ] || Player.status[ DISEASED ] )
						print2( "Phew! That's, er, smooth stuff!" );
					Player.status[ POISONED ] = 0;
					Player.status[ DISEASED ] = 0;
					showflags();
				}
				break;

			case 'c':
				if ( Player.cash < 100 )
				{
					print1( "Whatta feeb!" );
					print2( "Outta my establishment.... Now!" );
					p_damage( random_range( 20 ), UNSTOPPABLE, "Riley's right cross" );
					morewait();
				}
				else
				{
					Player.cash -= 100;
					dataprint();
					print1( "'What a guy!'" ); morewait();
					print2( "'Hey, thanks, fella.'" ); morewait();
					print3( "'Make mine a double...'" ); morewait();
					clearmsg();
					switch( random_range( 4 ))
					{
						case 0:
							print1( "'You are a real pal. Say, have you heard.... " );
							hint();
							break;

						case 1:
							print1( "A wandering priest of Dionysus blesses you..." );
							if (( Player.patron == ODIN ) || ( Player.patron == ATHENA ))
								Player.alignment++;
							else if (( Player.patron == HECATE ) || ( Player.patron == SET ))
								Player.alignment--;
							else if ( Player.alignment > 0 )
								Player.alignment--;
							else
								Player.alignment++;
							break;

						case 2:
							print1( "A thirsty bard promises to put your name in a song!" );
							gain_experience( 20 );
							break;

						case 3:
							print1( "Riley draws you a shot of his 'special reserve'" );
							print2( "Drink it [yn]?" );
							if ( ynq2() == 'y' )
							{
								if ( Player.con < random_range( 20 ))
								{
									print1( "<cough> Quite a kick!" );
									print2( "You feel a fiery warmth in your tummy...." );
									Player.con++;
									Player.maxcon++;
								}
								else
									print2( "You toss it back nonchalantly." );
							}
					}		
				}
				break;

			case 'd':
				if ( Player.cash < 25 )
					print2( "Pay in advance, mac!" );
				else
				{
					Player.cash -= 25;
					print2( "How about a shot o' the dew for a nightcap?" );
					morewait();
					Time += ( 6 + random_range( 4 )) * 60;
					Player.status[ POISONED ] = 0;
					Player.status[ DISEASED ] = 0;
					Player.food = 40;
					/* reduce temporary stat gains to max stat levels */
					toggle_item_use( TRUE );
					Player.str = min( Player.str, Player.maxstr );
					Player.con = min( Player.con, Player.maxcon );
					Player.agi = min( Player.agi, Player.maxagi );
					Player.dex = min( Player.dex, Player.maxdex );
					Player.iq = min( Player.iq, Player.maxiq );
					Player.pow = min( Player.pow, Player.maxpow );
					toggle_item_use( FALSE );
					timeprint();
					dataprint();
					showflags();
					print1( "The next day....." );
					if ( hour() > 10 )
						print2( "Oh my! You overslept!" );
				}
				break;
				
			default:
				print2( "So? Just looking? Go on!" );
				break;
		}
	}
	else
		print2( "The pub don't open til dark, fella." );
	xredraw();
}
#undef hinthour

void l_alchemist( void )
{
	int i, done = FALSE, mlevel;
	char response;
	pob obj;

	print1( "Ambrosias' Potions et cie." );
	if ( nighttime() ) 
		print2( "Ambrosias doesn't seem to be in right now." );
	else while ( !done )
	{
		morewait();
		clearmsg();
		print1( "a: Sell monster components." );
		print2( "b: Pay for transformation." );
		print3( "ESCAPE: Leave this place." );
		response = (char)mcigetc();
		if ( response == 'a' )
		{
			clearmsg();
			done = TRUE;
			i = o_getitem( C_CROSS );
			if (( i != ABORT ) && ( Player.possessions[ i ] != NULL ))
			{
				obj = Player.possessions[ i ];
				if ( Monsters[ obj->charge ].transformid == -1 )
				{
					print1( "I don't want such a thing." );
					if ( obj->basevalue > 0 )
						print2( "You might be able to sell it to someone else, though." );
				}
				else
				{
					clearmsg();
					print1( "I'll give you " );
					mnumprint( obj->basevalue / 3 );
					nprint1( "Au for it. Take it? [yn] " );
					if ( ynq1() == 'y' )
					{
						Player.cash += ( obj->basevalue / 3 );
						conform_lost_objects( 1, obj );
					}
					else
						print2( "Well, keep the smelly old thing, then!" );
				}
			}
			else
				print2( "So nu?" );
		}
		else if ( response == 'b' )
		{
			clearmsg();
			done = TRUE;
			i = o_getitem( C_CROSS );
			if (( i != ABORT ) && ( Player.possessions[ i ] != NULL ))
			{
				obj = Player.possessions[ i ];
				if ( Monsters[ obj->charge ].transformid == -1 )
					print1( "Oy vey! You want me to transform such a thing?" );
				else
				{
					mlevel = Monsters[ obj->charge ].level;
					print1( "It'll cost you " );
					mnumprint( max( 10, obj->basevalue * 2 ));
					nprint1( "Au for the transformation. Pay it? [yn] " );
					if ( ynq1() == 'y' )
					{
						if ( Player.cash < max( 10, obj->basevalue * 2 ))
							print2( "You can't afford it!" );
						else
						{
							print1( "Voila! A tap of the Philosopher's Stone..." );
							Player.cash -= max( 10, obj->basevalue * 2 );
							*obj = Objects[ Monsters[ obj->charge ].transformid ];
							if (( obj->id >= STICKID ) && ( obj->id < STICKID + NUMSTICKS ))
								obj->charge = 20;
							if ( obj->plus == 0 )
								obj->plus = mlevel;
							if ( obj->blessing == 0 )
								obj->blessing = 1;
						}
					}
					else
						print2( "I don't need your business, anyhow." );
				}
			}
			else
				print2( "So nu?" );
		}
		else if ( response == ESCAPE )
			done = TRUE;
	}
}

void l_dpw( void )
{
	print1( "Rampart Department of Public Works." );
	if ( Date - LastDay < 7 )
		print2( "G'wan! Get a job!" );
	else if ( Player.cash < 100 )
	{
		print2( "Do you want to go on the dole? [yn] " );
		if ( ynq2() == 'y' )
		{
			print1( "Well, ok, but spend it wisely." );
			print2( "Please enter your name for our records." );
			if ( strcmp( Player.name, msgscanstring() ) != 0 )
			{
				print3( "Aha! Welfare Fraud! It's off to gaol for you, lout!" );
				morewait();
				send_to_jail();
			}
			else
			{
				print1( "Here's your handout, layabout!" );
				LastDay = Date;
				Player.cash = 99;
				dataprint();
			}
		}
	}
	else
		print2( "You're too well off for us to help you!" );
}

void l_library( void )
{
	char response;
	int done = FALSE, fee = 1000;

	print1( "Rampart Public Library." );
	if ( nighttime() )
		print2( "CLOSED" );
	else
	{
		morewait();
		print1( "Library Research Fee: 1000Au." );
		if ( Player.maxiq < 18 )
		{
			print2( "The Rampart student aid system has arranged a grant!" );
			morewait();
			clearmsg();
			print1( "Your revised fee is: " );
			mnumprint( fee = max( 50, 1000 - ( 18 - Player.maxiq ) * 125 ));
			nprint1( "Au." );
		}
		morewait();
		while( !done )
		{
			print1( "Pay the fee? [yn] " );
			if ( ynq1() == 'y' )
			{
				if ( Player.cash < fee )
				{
					print2( "No payee, No studee." );
					done = TRUE;
				}
				else
				{
					Player.cash -= fee;
					dataprint();
					menuclear();
					menuprint( "Peruse a scroll:\n" );
					menuprint( "a: Omegan Theology\n" );
					menuprint( "b: Guide to Rampart\n" );
					menuprint( "c: High Magick\n" );
					menuprint( "d: Odd Uncatalogued Document\n" );
					menuprint( "e: Attempt Advanced Research\n" );
					menuprint( "ESCAPE: Leave this font of learning.\n" );
					showmenu();
					response = (char)mcigetc();
					if ( response == 'a' )
					{
						print1( "You unfurl an ancient, yellowing scroll..." );
						morewait();
						theologyfile();
					}
					else if ( response == 'b' )
					{
						print1( "You unroll a slick four-color document..." );
						morewait();
						cityguidefile();
					}
					else if ( response == 'c' )
					{
						print1( "This scroll is written in a strange magical script..." );
						morewait();
						wishfile();
					}
					else if ( response == 'd' )
					{
						print1( "You find a strange document, obviously misfiled" );
						print2( "under the heading 'acrylic fungus painting technique'" );
						morewait();
						adeptfile();
					}
					else if ( response == 'e' )
					{
						if ( random_range( 30 ) > Player.iq )
						{
							print2( "You feel more knowledgeable!" );
							Player.iq++;
							Player.maxiq++;
							dataprint();
							if ( Player.maxiq < 19 &&
								 fee != max( 50, 1000 - ( 18 - Player.maxiq ) * 125 ))
							{
								morewait();
								clearmsg();
								print1( "Your revised fee is: " );
								mnumprint( fee = max( 50,1000 - ( 18 - Player.maxiq ) * 125 ));
								nprint1( "Au." );
								morewait();
							}
						}
						else
						{
							clearmsg1();
							print1( "You find advice in an ancient tome: " );
							morewait();
							hint();
							morewait();
						}
					}
					else
						done = ( response == ESCAPE );
				}
				xredraw();
			}
			else
			{
				done = TRUE;
				print2( "You philistine!" );
			}
		}
	}
}

void l_pawn_shop( void )
{
	int i, j, k, limit, number, done = FALSE;
	char item, action;

	if ( nighttime() )
		print1( "Shop Closed: Have a Nice (K)Night" );
	else {
		limit = min( 5, Date - Pawndate );
		Pawndate = Date;
		for( k = 0; k < limit; k++ )
		{
			if ( Pawnitems[ 0 ] != NULL )
			{
				if ( Objects[ Pawnitems[ 0 ]->id ].uniqueness > UNIQUE_UNMADE )
					Objects[ Pawnitems[ 0 ]->id ].uniqueness = UNIQUE_UNMADE;
				/* could turn up anywhere, really :) */
				free( (char *)Pawnitems[ 0 ] );
				Pawnitems[ 0 ] = NULL;
			}
			for ( i = 0; i < PAWNITEMS - 1; i++ )
				Pawnitems[ i ] = Pawnitems[ i + 1 ];
			Pawnitems[ PAWNITEMS - 1 ] = NULL;
			for ( i = 0; i < PAWNITEMS; i++ )
				if ( Pawnitems[ i ] == NULL )
				do {
					if ( Pawnitems[ i ] != NULL )
						free( Pawnitems[ i ] );
					do Pawnitems[ i ] = create_object( 5 );
					while ( !Pawnitems[ i ] );
					Pawnitems[ i ]->known = 2;
				} while (( Pawnitems[ i ]->objchar == CASH ) ||
						 ( Pawnitems[ i ]->objchar >= CASH_FIRST &&
						   Pawnitems[ i ]->objchar <= CASH_LAST ) ||
						 ( Pawnitems[ i ]->objchar == ARTIFACT ) ||
						 ( Pawnitems[ i ]->objchar >= ARTIFACT_FIRST &&
						   Pawnitems[ i ]->objchar <= ARTIFACT_LAST ) ||
						 ( true_item_value( Pawnitems[ i ] ) <= 0 ));
		}
		while ( !done )
		{
			print1( "Knight's Pawn Shop:" );
			print2( "Buy item, Sell item, sell Pack contents, Leave [b,s,p,ESCAPE] " );
			menuclear();
			for( i = 0; i < PAWNITEMS; i++ ) 
				if ( Pawnitems[ i ] != NULL )
				{
					strcpy( Str3, " :" );
					Str3[ 0 ] = i + 'a';
					strcat( Str3, itemid( Pawnitems[ i ] ));
					menuprint( Str3 );
					menuprint( "\n" );
				}
			showmenu();
			action = (char)mcigetc();
			if ( action == ESCAPE )
				done = TRUE;
			else if ( action == 'b' )
			{
				print2( "Purchase which item? [ESCAPE to quit] " );
				item = ' ';
				while (( item != ESCAPE ) &&
					   (( item < 'a' ) || ( item >= 'a' + PAWNITEMS )))
				item = (char)mcigetc();
				if ( item != ESCAPE )
				{
					i = item - 'a';
					if ( Pawnitems[ i ] == NULL )
						print3( "No such item!" );
					else if ( true_item_value( Pawnitems[ i ] ) <= 0 )
					{
						print1( "Hmm, how did that junk get on my shelves?" );
						print2( "I'll just remove it." );
						free( (char *)Pawnitems[ i ] );
						Pawnitems[ i ] = NULL;
					}
					else
					{
						clearmsg();
						print1( "The low, low, cost is: " );
						mlongprint( Pawnitems[ i ]->number * true_item_value( Pawnitems[ i ] ));
						nprint1( " Buy it? [ynq] " );
						if ( ynq1() == 'y' )
						{
							if ( Player.cash < Pawnitems[ i ]->number *
											   true_item_value( Pawnitems[ i ] ))
							{
								print2( "No credit! Gwan, Beat it!" );
								morewait();
							}
							else
							{
								Player.cash -= Pawnitems[ i ]->number *
											   true_item_value( Pawnitems[ i ] );
								Objects[ Pawnitems[ i ]->id ].known = 1;
								gain_item( Pawnitems[ i ] );
								Pawnitems[ i ] = NULL;
							}
						}
					}
				}
			}
			else if ( action == 's' )
			{
				menuclear();
				print2( "Sell which item: " );
				i = o_getitem( NULL_ITEM );
				if (( i != ABORT ) && ( Player.possessions[ i ] != NULL ))
				{
					if ( cursed( Player.possessions[ i ] ))
					{
						print1( "No loans on cursed items! I been burned before...." );
						morewait();
					}
					else if ( true_item_value( Player.possessions[ i ] ) <= 0 )
					{
						print1( "That looks like a worthless piece of junk to me." );
						morewait();
					}
					else
					{
						clearmsg();
						print1( "You can get " );
						mlongprint( item_value( Player.possessions[ i ] ) / 2 );
						nprint1( "Au each. Sell [yn]? " );
						if ( ynq1() == 'y' )
						{
							number = getnumber( Player.possessions[ i ]->number );
							if (( number >= Player.possessions[ i ]->number ) &&
								Player.possessions[ i ]->used )
							{
								Player.possessions[ i ]->used = FALSE;
								item_use( Player.possessions[ i ] );
							}
							Player.cash += number * item_value( Player.possessions[ i ] ) / 2;
							free( (char *)Pawnitems[ 0 ] );
							for( j = 0; j < PAWNITEMS - 1; j++ )
								Pawnitems[ j ] = Pawnitems[ j + 1 ];
							Pawnitems[ PAWNITEMS - 1 ] = ( (pob)checkmalloc( sizeof( objtype )));
							*( Pawnitems[ PAWNITEMS - 1 ] ) = *( Player.possessions[ i ] );
							Pawnitems[ PAWNITEMS - 1 ]->number = number;
							Pawnitems[ PAWNITEMS - 1 ]->known = 2;	
							dispose_lost_objects( number, Player.possessions[ i ] );
							dataprint();
						}
					}
				}
			}
			else if ( action == 'p' )
			{
				for( i = 0; i < Player.packptr; i++ )
				{
					if ( Player.pack[ i ]->blessing > -1 &&
						 true_item_value( Player.pack[ i ] ) > 0 )
					{
						clearmsg();
						print1( "Sell " );
						nprint1( itemid( Player.pack[ i ] ));
						nprint1( " for " );
						mlongprint( item_value( Player.pack[ i ] ) / 2 );
						nprint1( "Au each? [yn] " );
						if ( ynq1() == 'y' )
						{
							number = getnumber( Player.pack[ i ]->number );
							if ( number > 0 )
							{
								Player.cash += number * item_value( Player.pack[ i ] ) / 2;
								free( (char *)Pawnitems[ 0 ] );
								for( j = 0; j < PAWNITEMS - 1; j++ )
									Pawnitems[ j ] = Pawnitems[ j + 1 ];
								Pawnitems[ PAWNITEMS - 1 ] =
									( (pob)checkmalloc( sizeof( objtype )));
								*( Pawnitems[ PAWNITEMS - 1 ] ) = *( Player.pack[ i ] );
								Pawnitems[ PAWNITEMS - 1 ]->number = number;
								Pawnitems[ PAWNITEMS - 1 ]->known = 2;
								Player.pack[ i ]->number -= number;
								if ( Player.pack[ i ]->number < 1 )
								{
									free( (char *)Player.pack[ i ] );
									Player.pack[ i ] = NULL;
								}
								dataprint();
							}
						}
					}
				}
				fixpack();
			}
		}
	}
	calc_melee();
	xredraw();
}
