/* truetiles.c */
/* some functions to select the correct tile of a set of, for example, walls */

#include "curses.h"
#include "glob.h"
#include "ochars.h"

void UserProgramAbout( void );
short old_to_new_locchar( short c, int aux );
void set_correct_city_wall( int i, int j, plv Level );
void set_correct_hedge( int i, int j, plv Level );
void set_correct_pool( int i, int j, plv Level );
void set_correct_tile( int i, int j, plv Level );
short assign_wall_char( short charvec );
void fixup_walls( int x, int y, plv Level );
void fixup_hedges( int x, int y, plv Level );
void fixup_pools( int x, int y, plv Level );

/* just stuck in here because all this stuff is mine - wss */
void UserProgramAbout( void )
{
	show_license();
}

short old_to_new_locchar( short c, int aux )
{
	switch( c & 0xff )
	{
		/*
		case 0xff & SPACE:			c = SPACE;			break;
		*/
		case 0xff & WALL:			c = WALL;			break;
		case 0xff & PORTCULLIS:		c = PORTCULLIS;		break;
		case 0xff & OPEN_DOOR:		c = MY_OPEN_DOOR;	break;
		case 0xff & CLOSED_DOOR:	c = MY_CLOSED_DOOR;	break;
		case 0xff & WHIRLWIND:		c = MY_WHIRLWIND;	break;
		case 0xff & ABYSS:			c = MY_ABYSS;		break;
		case 0xff & VOID_CHAR:		c = MY_VOID;		break;
		case 0xff & LAVA:			c = MY_LAVA;		break;
		case 0xff & HEDGE:			c = HEDGE_WHOLE;	break;
		case 0xff & WATER:			c = POOL_WHOLE;		break;
		case 0xff & FIRE:			c = MY_FIRE;		break;
		case 0xff & TRAP:			c = C_TRAP;			break;
		case 0xff & LIFT:			c = MY_LIFT;		break;
		case 0xff & STAIRS_UP:		c = MY_STAIRS_UP;	break;
		case 0xff & STAIRS_DOWN:	c = MY_STAIRS_DOWN;	break;
		case 0xff & FLOOR:			c = GROUND;			break;
		case 0xff & STATUE:			c = MY_STATUE;		break;
		case 0xff & RUBBLE:			c = MY_RUBBLE;		break;
		case 0xff & ALTAR:
			switch ( aux )
			{
				case HECATE:	c = ALTAR_HECATE;	break;
				case ODIN:		c = ALTAR_ODIN;		break;
				case DESTINY:	c = ALTAR_DESTINY;	break;
				case SET:		c = ALTAR_SET;		break;
				case ATHENA:	c = ALTAR_ATHENA;	break;
				case DRUID:		c = ALTAR_DRUID;	break;
				default:		c = ALTAR_UNKNOWN;	break;
			};
			break;
	};
	
	return c;
}

void set_correct_hedge( int i, int j, plv Level )
{
	short c, locvec;
	int x, y;
	
	locvec = 0;
	for ( y = j-1; y < j+2; y++ )
		for ( x = i-1; x < i+2; x++ )
		{
			if ( x < 0 || y < 0 || x >= WIDTH || y >= LENGTH )
			{
				locvec <<= 1;
				locvec |= 0x01;
			}
			else if ( !( x == i && y == j ) )
			{
				locvec <<= 1;
				c = Level->site[ x ][ y ].locchar;
				/* treat secret walls as hedges in temples (for woodhenge) */
				if (( c >= HEDGE_FIRST && c <= HEDGE_LAST ) ||
					( loc_statusp( x, y, SECRET ) && Level->environment == E_TEMPLE ))
				{
					locvec |= 0x01;
				}
			}
		}
	
	switch( locvec )
	{
		case 0x0:
		case 0x1:
		case 0x4:
		case 0x5:
		case 0x20:
		case 0x21:
		case 0x24:
		case 0x25:
		case 0x80:
		case 0x81:
		case 0x84:
		case 0x85:
		case 0xa0:
		case 0xa1:
		case 0xa4:
		case 0xa5:
			Level->site[ i ][ j ].locchar = HEDGE_EFGH;
			break;
			
		case 0x2:
		case 0x3:
		case 0x6:
		case 0x7:
		case 0x22:
		case 0x23:
		case 0x26:
		case 0x27:
		case 0x82:
		case 0x83:
		case 0x86:
		case 0x87:
		case 0xa2:
		case 0xa3:
		case 0xa6:
		case 0xa7:
			Level->site[ i ][ j ].locchar = HEDGE_EFG;
			break;
			
		case 0x8:
		case 0x9:
		case 0xc:
		case 0xd:
		case 0x28:
		case 0x29:
		case 0x2c:
		case 0x2d:
		case 0x88:
		case 0x89:
		case 0x8c:
		case 0x8d:
		case 0xa8:
		case 0xa9:
		case 0xac:
		case 0xad:
			Level->site[ i ][ j ].locchar = HEDGE_EFH;
			break;
			
		case 0xa:
		case 0xe:
		case 0x2a:
		case 0x2e:
		case 0x8a:
		case 0x8e:
		case 0xaa:
		case 0xae:
			Level->site[ i ][ j ].locchar = HEDGE_DEF;
			break;
			
		case 0xb:
		case 0xf:
		case 0x2b:
		case 0x2f:
		case 0x8b:
		case 0x8f:
		case 0xab:
		case 0xaf:
			Level->site[ i ][ j ].locchar = HEDGE_EF;
			break;
			
		case 0x10:
		case 0x11:
		case 0x14:
		case 0x15:
		case 0x30:
		case 0x31:
		case 0x34:
		case 0x35:
		case 0x90:
		case 0x91:
		case 0x94:
		case 0x95:
		case 0xb0:
		case 0xb1:
		case 0xb4:
		case 0xb5:
			Level->site[ i ][ j ].locchar = HEDGE_EGH;
			break;
		
		case 0x12:
		case 0x13:
		case 0x32:
		case 0x33:
		case 0x92:
		case 0x93:
		case 0xb2:
		case 0xb3:
			Level->site[ i ][ j ].locchar = HEDGE_CEG;
			break;
			
		case 0x16:
		case 0x17:
		case 0x36:
		case 0x37:
		case 0x96:
		case 0x97:
		case 0xb6:
		case 0xb7:
			Level->site[ i ][ j ].locchar = HEDGE_EG;
			break;
		
		case 0x18:
		case 0x19:
		case 0x1c:
		case 0x1d:
		case 0x38:
		case 0x39:
		case 0x3c:
		case 0x3d:
		case 0x98:
		case 0x99:
		case 0x9c:
		case 0x9d:
		case 0xb8:
		case 0xb9:
		case 0xbc:
		case 0xbd:
			Level->site[ i ][ j ].locchar = HEDGE_EH;
			break;
			
		case 0x1a:
		case 0x3a:
		case 0x9a:
		case 0xba:
			Level->site[ i ][ j ].locchar = HEDGE_CDE;
			break;
			
		case 0x1b:
		case 0x3b:
		case 0x9b:
		case 0xbb:
			Level->site[ i ][ j ].locchar = HEDGE_CE;
			break;
			
		case 0x1e:
		case 0x3e:
		case 0x9e:
		case 0xbe:
			Level->site[ i ][ j ].locchar = HEDGE_DE;
			break;
			
		case 0x1f:
		case 0x3f:
		case 0x9f:
		case 0xbf:
			Level->site[ i ][ j ].locchar = HEDGE_E;
			break;
			
		case 0x40:
		case 0x41:
		case 0x44:
		case 0x45:
		case 0x60:
		case 0x61:
		case 0x64:
		case 0x65:
		case 0xc0:
		case 0xc1:
		case 0xc4:
		case 0xc5:
		case 0xe0:
		case 0xe1:
		case 0xe4:
		case 0xe5:
			Level->site[ i ][ j ].locchar = HEDGE_FGH;
			break;
			
		case 0x42:
		case 0x43:
		case 0x46:
		case 0x47:
		case 0x62:
		case 0x63:
		case 0x66:
		case 0x67:
		case 0xc2:
		case 0xc3:
		case 0xc6:
		case 0xc7:
		case 0xe2:
		case 0xe3:
		case 0xe6:
		case 0xe7:
			Level->site[ i ][ j ].locchar = HEDGE_FG;
			break;
			
		case 0x48:
		case 0x49:
		case 0x4c:
		case 0x4d:
		case 0xc8:
		case 0xc9:
		case 0xcc:
		case 0xcd:
			Level->site[ i ][ j ].locchar = HEDGE_BFH;
			break;
			
		case 0x4a:
		case 0x4e:
		case 0xca:
		case 0xce:
			Level->site[ i ][ j ].locchar = HEDGE_BDF;
			break;
			
		case 0x4b:
		case 0x4f:
		case 0xcb:
		case 0xcf:
			Level->site[ i ][ j ].locchar = HEDGE_BF;
			break;
			
		case 0x50:
		case 0x51:
		case 0x54:
		case 0x55:
		case 0x70:
		case 0x71:
		case 0x74:
		case 0x75:
			Level->site[ i ][ j ].locchar = HEDGE_AGH;
			break;
			
		case 0x52:
		case 0x53:
		case 0x72:
		case 0x73:
			Level->site[ i ][ j ].locchar = HEDGE_ACG;
			break;
			
		case 0x56:
		case 0x57:
		case 0x76:
		case 0x77:
			Level->site[ i ][ j ].locchar = HEDGE_AG;
			break;
			
		case 0x58:
		case 0x59:
		case 0x5c:
		case 0x5d:
			Level->site[ i ][ j ].locchar = HEDGE_ABH;
			break;
			
		case 0x5a:
			Level->site[ i ][ j ].locchar = HEDGE_ABCD;
			break;
			
		case 0x5b:
			Level->site[ i ][ j ].locchar = HEDGE_ABC;
			break;
			
		case 0x5e:
			Level->site[ i ][ j ].locchar = HEDGE_ABD;
			break;
			
		case 0x5f:
			Level->site[ i ][ j ].locchar = HEDGE_AB;
			break;
			
		case 0x68:
		case 0x69:
		case 0x6c:
		case 0x6d:
		case 0xe8:
		case 0xe9:
		case 0xec:
		case 0xed:
			Level->site[ i ][ j ].locchar = HEDGE_FH;
			break;
			
		case 0x6a:
		case 0x6e:
		case 0xea:
		case 0xee:
			Level->site[ i ][ j ].locchar = HEDGE_DF;
			break;
			
		case 0x6b:
		case 0x6f:
		case 0xeb:
		case 0xef:
			Level->site[ i ][ j ].locchar = HEDGE_F;
			break;
			
		case 0x78:
		case 0x79:
		case 0x7c:
		case 0x7d:
			Level->site[ i ][ j ].locchar = HEDGE_AH;
			break;
			
		case 0x7a:
			Level->site[ i ][ j ].locchar = HEDGE_ACD;
			break;
			
		case 0x7b:
			Level->site[ i ][ j ].locchar = HEDGE_AC;
			break;
			
		case 0x7e:
			Level->site[ i ][ j ].locchar = HEDGE_AD;
			break;
			
		case 0x7f:
			Level->site[ i ][ j ].locchar = HEDGE_A;
			break;
			
		case 0xd0:
		case 0xd1:
		case 0xd4:
		case 0xd5:
		case 0xf0:
		case 0xf1:
		case 0xf4:
		case 0xf5:
			Level->site[ i ][ j ].locchar = HEDGE_GH;
			break;
			
		case 0xd2:
		case 0xd3:
		case 0xf2:
		case 0xf3:
			Level->site[ i ][ j ].locchar = HEDGE_CG;
			break;
			
		case 0xd6:
		case 0xd7:
		case 0xf6:
		case 0xf7:
			Level->site[ i ][ j ].locchar = HEDGE_G;
			break;
			
		case 0xd8:
		case 0xd9:
		case 0xdc:
		case 0xdd:
			Level->site[ i ][ j ].locchar = HEDGE_BH;
			break;
			
		case 0xda:
			Level->site[ i ][ j ].locchar = HEDGE_BCD;
			break;
			
		case 0xdb:
			Level->site[ i ][ j ].locchar = HEDGE_BC;
			break;
			
		case 0xde:
			Level->site[ i ][ j ].locchar = HEDGE_BD;
			break;
			
		case 0xdf:
			Level->site[ i ][ j ].locchar = HEDGE_B;
			break;
			
		case 0xf8:
		case 0xf9:
		case 0xfc:
		case 0xfd:
			Level->site[ i ][ j ].locchar = HEDGE_H;
			break;
			
		case 0xfa:
			Level->site[ i ][ j ].locchar = HEDGE_CD;
			break;
			
		case 0xfb:
			Level->site[ i ][ j ].locchar = HEDGE_C;
			break;
			
		case 0xfe:
			Level->site[ i ][ j ].locchar = HEDGE_D;
			break;
		
		case 0xff:
		default:
			Level->site[ i ][ j ].locchar = HEDGE_WHOLE;
	}
}

void set_correct_pool( int i, int j, plv Level )
{
	short c, locvec;
	int x, y;
	
	locvec = 0;
	for ( y = j-1; y < j+2; y++ )
		for ( x = i-1; x < i+2; x++ )
		{
			if ( x < 0 || y < 0 || x >= WIDTH || y >= LENGTH )
			{
				locvec <<= 1;
				locvec |= 0x01;
			}
			else if ( !( x == i && y == j ) )
			{
				locvec <<= 1;
				c = Level->site[ x ][ y ].locchar;
				if ( c >= POOL_FIRST && c <= POOL_LAST )
				{
					locvec |= 0x01;
				}
			}
		}
	
	switch( locvec )
	{
		case 0x0:
		case 0x1:
		case 0x4:
		case 0x5:
		case 0x20:
		case 0x21:
		case 0x24:
		case 0x25:
		case 0x80:
		case 0x81:
		case 0x84:
		case 0x85:
		case 0xa0:
		case 0xa1:
		case 0xa4:
		case 0xa5:
			Level->site[ i ][ j ].locchar = POOL_EFGH;
			break;
			
		case 0x2:
		case 0x3:
		case 0x6:
		case 0x7:
		case 0x22:
		case 0x23:
		case 0x26:
		case 0x27:
		case 0x82:
		case 0x83:
		case 0x86:
		case 0x87:
		case 0xa2:
		case 0xa3:
		case 0xa6:
		case 0xa7:
			Level->site[ i ][ j ].locchar = POOL_EFG;
			break;
			
		case 0x8:
		case 0x9:
		case 0xc:
		case 0xd:
		case 0x28:
		case 0x29:
		case 0x2c:
		case 0x2d:
		case 0x88:
		case 0x89:
		case 0x8c:
		case 0x8d:
		case 0xa8:
		case 0xa9:
		case 0xac:
		case 0xad:
			Level->site[ i ][ j ].locchar = POOL_EFH;
			break;
			
		case 0xa:
		case 0xe:
		case 0x2a:
		case 0x2e:
		case 0x8a:
		case 0x8e:
		case 0xaa:
		case 0xae:
			Level->site[ i ][ j ].locchar = POOL_DEF;
			break;
			
		case 0xb:
		case 0xf:
		case 0x2b:
		case 0x2f:
		case 0x8b:
		case 0x8f:
		case 0xab:
		case 0xaf:
			Level->site[ i ][ j ].locchar = POOL_EF;
			break;
			
		case 0x10:
		case 0x11:
		case 0x14:
		case 0x15:
		case 0x30:
		case 0x31:
		case 0x34:
		case 0x35:
		case 0x90:
		case 0x91:
		case 0x94:
		case 0x95:
		case 0xb0:
		case 0xb1:
		case 0xb4:
		case 0xb5:
			Level->site[ i ][ j ].locchar = POOL_EGH;
			break;
		
		case 0x12:
		case 0x13:
		case 0x32:
		case 0x33:
		case 0x92:
		case 0x93:
		case 0xb2:
		case 0xb3:
			Level->site[ i ][ j ].locchar = POOL_CEG;
			break;
			
		case 0x16:
		case 0x17:
		case 0x36:
		case 0x37:
		case 0x96:
		case 0x97:
		case 0xb6:
		case 0xb7:
			Level->site[ i ][ j ].locchar = POOL_EG;
			break;
		
		case 0x18:
		case 0x19:
		case 0x1c:
		case 0x1d:
		case 0x38:
		case 0x39:
		case 0x3c:
		case 0x3d:
		case 0x98:
		case 0x99:
		case 0x9c:
		case 0x9d:
		case 0xb8:
		case 0xb9:
		case 0xbc:
		case 0xbd:
			Level->site[ i ][ j ].locchar = POOL_EH;
			break;
			
		case 0x1a:
		case 0x3a:
		case 0x9a:
		case 0xba:
			Level->site[ i ][ j ].locchar = POOL_CDE;
			break;
			
		case 0x1b:
		case 0x3b:
		case 0x9b:
		case 0xbb:
			Level->site[ i ][ j ].locchar = POOL_CE;
			break;
			
		case 0x1e:
		case 0x3e:
		case 0x9e:
		case 0xbe:
			Level->site[ i ][ j ].locchar = POOL_DE;
			break;
			
		case 0x1f:
		case 0x3f:
		case 0x9f:
		case 0xbf:
			Level->site[ i ][ j ].locchar = POOL_E;
			break;
			
		case 0x40:
		case 0x41:
		case 0x44:
		case 0x45:
		case 0x60:
		case 0x61:
		case 0x64:
		case 0x65:
		case 0xc0:
		case 0xc1:
		case 0xc4:
		case 0xc5:
		case 0xe0:
		case 0xe1:
		case 0xe4:
		case 0xe5:
			Level->site[ i ][ j ].locchar = POOL_FGH;
			break;
			
		case 0x42:
		case 0x43:
		case 0x46:
		case 0x47:
		case 0x62:
		case 0x63:
		case 0x66:
		case 0x67:
		case 0xc2:
		case 0xc3:
		case 0xc6:
		case 0xc7:
		case 0xe2:
		case 0xe3:
		case 0xe6:
		case 0xe7:
			Level->site[ i ][ j ].locchar = POOL_FG;
			break;
			
		case 0x48:
		case 0x49:
		case 0x4c:
		case 0x4d:
		case 0xc8:
		case 0xc9:
		case 0xcc:
		case 0xcd:
			Level->site[ i ][ j ].locchar = POOL_BFH;
			break;
			
		case 0x4a:
		case 0x4e:
		case 0xca:
		case 0xce:
			Level->site[ i ][ j ].locchar = POOL_BDF;
			break;
			
		case 0x4b:
		case 0x4f:
		case 0xcb:
		case 0xcf:
			Level->site[ i ][ j ].locchar = POOL_BF;
			break;
			
		case 0x50:
		case 0x51:
		case 0x54:
		case 0x55:
		case 0x70:
		case 0x71:
		case 0x74:
		case 0x75:
			Level->site[ i ][ j ].locchar = POOL_AGH;
			break;
			
		case 0x52:
		case 0x53:
		case 0x72:
		case 0x73:
			Level->site[ i ][ j ].locchar = POOL_ACG;
			break;
			
		case 0x56:
		case 0x57:
		case 0x76:
		case 0x77:
			Level->site[ i ][ j ].locchar = POOL_AG;
			break;
			
		case 0x58:
		case 0x59:
		case 0x5c:
		case 0x5d:
			Level->site[ i ][ j ].locchar = POOL_ABH;
			break;
			
		case 0x5a:
			Level->site[ i ][ j ].locchar = POOL_ABCD;
			break;
			
		case 0x5b:
			Level->site[ i ][ j ].locchar = POOL_ABC;
			break;
			
		case 0x5e:
			Level->site[ i ][ j ].locchar = POOL_ABD;
			break;
			
		case 0x5f:
			Level->site[ i ][ j ].locchar = POOL_AB;
			break;
			
		case 0x68:
		case 0x69:
		case 0x6c:
		case 0x6d:
		case 0xe8:
		case 0xe9:
		case 0xec:
		case 0xed:
			Level->site[ i ][ j ].locchar = POOL_FH;
			break;
			
		case 0x6a:
		case 0x6e:
		case 0xea:
		case 0xee:
			Level->site[ i ][ j ].locchar = POOL_DF;
			break;
			
		case 0x6b:
		case 0x6f:
		case 0xeb:
		case 0xef:
			Level->site[ i ][ j ].locchar = POOL_F;
			break;
			
		case 0x78:
		case 0x79:
		case 0x7c:
		case 0x7d:
			Level->site[ i ][ j ].locchar = POOL_AH;
			break;
			
		case 0x7a:
			Level->site[ i ][ j ].locchar = POOL_ACD;
			break;
			
		case 0x7b:
			Level->site[ i ][ j ].locchar = POOL_AC;
			break;
			
		case 0x7e:
			Level->site[ i ][ j ].locchar = POOL_AD;
			break;
			
		case 0x7f:
			Level->site[ i ][ j ].locchar = POOL_A;
			break;
			
		case 0xd0:
		case 0xd1:
		case 0xd4:
		case 0xd5:
		case 0xf0:
		case 0xf1:
		case 0xf4:
		case 0xf5:
			Level->site[ i ][ j ].locchar = POOL_GH;
			break;
			
		case 0xd2:
		case 0xd3:
		case 0xf2:
		case 0xf3:
			Level->site[ i ][ j ].locchar = POOL_CG;
			break;
			
		case 0xd6:
		case 0xd7:
		case 0xf6:
		case 0xf7:
			Level->site[ i ][ j ].locchar = POOL_G;
			break;
			
		case 0xd8:
		case 0xd9:
		case 0xdc:
		case 0xdd:
			Level->site[ i ][ j ].locchar = POOL_BH;
			break;
			
		case 0xda:
			Level->site[ i ][ j ].locchar = POOL_BCD;
			break;
			
		case 0xdb:
			Level->site[ i ][ j ].locchar = POOL_BC;
			break;
			
		case 0xde:
			Level->site[ i ][ j ].locchar = POOL_BD;
			break;
			
		case 0xdf:
			Level->site[ i ][ j ].locchar = POOL_B;
			break;
			
		case 0xf8:
		case 0xf9:
		case 0xfc:
		case 0xfd:
			Level->site[ i ][ j ].locchar = POOL_H;
			break;
			
		case 0xfa:
			Level->site[ i ][ j ].locchar = POOL_CD;
			break;
			
		case 0xfb:
			Level->site[ i ][ j ].locchar = POOL_C;
			break;
			
		case 0xfe:
			Level->site[ i ][ j ].locchar = POOL_D;
			break;
		
		case 0xff:
		default:
			Level->site[ i ][ j ].locchar = POOL_WHOLE;
	}
}

short assign_wall_char( short charvec )
{
	switch( charvec )
	{
		case 0x0:
		case 0x1:
		case 0x4:
		case 0x5:
		case 0x20:
		case 0x21:
		case 0x24:
		case 0x25:
		case 0x80:
		case 0x81:
		case 0x84:
		case 0x85:
		case 0xa0:
		case 0xa1:
		case 0xa4:
		case 0xa5:
			return WALL_EFGH;
			break;
			
		case 0x2:
		case 0x3:
		case 0x6:
		case 0x7:
		case 0x22:
		case 0x23:
		case 0x26:
		case 0x27:
		case 0x82:
		case 0x83:
		case 0x86:
		case 0x87:
		case 0xa2:
		case 0xa3:
		case 0xa6:
		case 0xa7:
			return WALL_EFG;
			break;
			
		case 0x8:
		case 0x9:
		case 0xc:
		case 0xd:
		case 0x28:
		case 0x29:
		case 0x2c:
		case 0x2d:
		case 0x88:
		case 0x89:
		case 0x8c:
		case 0x8d:
		case 0xa8:
		case 0xa9:
		case 0xac:
		case 0xad:
			return WALL_EFH;
			break;
			
		case 0xa:
		case 0xe:
		case 0x2a:
		case 0x2e:
		case 0x8a:
		case 0x8e:
		case 0xaa:
		case 0xae:
			return WALL_DEF;
			break;
			
		case 0xb:
		case 0xf:
		case 0x2b:
		case 0x2f:
		case 0x8b:
		case 0x8f:
		case 0xab:
		case 0xaf:
			return WALL_EF;
			break;
			
		case 0x10:
		case 0x11:
		case 0x14:
		case 0x15:
		case 0x30:
		case 0x31:
		case 0x34:
		case 0x35:
		case 0x90:
		case 0x91:
		case 0x94:
		case 0x95:
		case 0xb0:
		case 0xb1:
		case 0xb4:
		case 0xb5:
			return WALL_EGH;
			break;
		
		case 0x12:
		case 0x13:
		case 0x32:
		case 0x33:
		case 0x92:
		case 0x93:
		case 0xb2:
		case 0xb3:
			return WALL_CEG;
			break;
			
		case 0x16:
		case 0x17:
		case 0x36:
		case 0x37:
		case 0x96:
		case 0x97:
		case 0xb6:
		case 0xb7:
			return WALL_EG;
			break;
		
		case 0x18:
		case 0x19:
		case 0x1c:
		case 0x1d:
		case 0x38:
		case 0x39:
		case 0x3c:
		case 0x3d:
		case 0x98:
		case 0x99:
		case 0x9c:
		case 0x9d:
		case 0xb8:
		case 0xb9:
		case 0xbc:
		case 0xbd:
			return WALL_EH;
			break;
			
		case 0x1a:
		case 0x3a:
		case 0x9a:
		case 0xba:
			return WALL_CDE;
			break;
			
		case 0x1b:
		case 0x3b:
		case 0x9b:
		case 0xbb:
			return WALL_CE;
			break;
			
		case 0x1e:
		case 0x3e:
		case 0x9e:
		case 0xbe:
			return WALL_DE;
			break;
			
		case 0x1f:
		case 0x3f:
		case 0x9f:
		case 0xbf:
			return WALL_E;
			break;
			
		case 0x40:
		case 0x41:
		case 0x44:
		case 0x45:
		case 0x60:
		case 0x61:
		case 0x64:
		case 0x65:
		case 0xc0:
		case 0xc1:
		case 0xc4:
		case 0xc5:
		case 0xe0:
		case 0xe1:
		case 0xe4:
		case 0xe5:
			return WALL_FGH;
			break;
			
		case 0x42:
		case 0x43:
		case 0x46:
		case 0x47:
		case 0x62:
		case 0x63:
		case 0x66:
		case 0x67:
		case 0xc2:
		case 0xc3:
		case 0xc6:
		case 0xc7:
		case 0xe2:
		case 0xe3:
		case 0xe6:
		case 0xe7:
			return WALL_FG;
			break;
			
		case 0x48:
		case 0x49:
		case 0x4c:
		case 0x4d:
		case 0xc8:
		case 0xc9:
		case 0xcc:
		case 0xcd:
			return WALL_BFH;
			break;
			
		case 0x4a:
		case 0x4e:
		case 0xca:
		case 0xce:
			return WALL_BDF;
			break;
			
		case 0x4b:
		case 0x4f:
		case 0xcb:
		case 0xcf:
			return WALL_BF;
			break;
			
		case 0x50:
		case 0x51:
		case 0x54:
		case 0x55:
		case 0x70:
		case 0x71:
		case 0x74:
		case 0x75:
			return WALL_AGH;
			break;
			
		case 0x52:
		case 0x53:
		case 0x72:
		case 0x73:
			return WALL_ACG;
			break;
			
		case 0x56:
		case 0x57:
		case 0x76:
		case 0x77:
			return WALL_AG;
			break;
			
		case 0x58:
		case 0x59:
		case 0x5c:
		case 0x5d:
			return WALL_ABH;
			break;
			
		case 0x5a:
			return WALL_ABCD;
			break;
			
		case 0x5b:
			return WALL_ABC;
			break;
			
		case 0x5e:
			return WALL_ABD;
			break;
			
		case 0x5f:
			return WALL_AB;
			break;
			
		case 0x68:
		case 0x69:
		case 0x6c:
		case 0x6d:
		case 0xe8:
		case 0xe9:
		case 0xec:
		case 0xed:
			return WALL_FH;
			break;
			
		case 0x6a:
		case 0x6e:
		case 0xea:
		case 0xee:
			return WALL_DF;
			break;
			
		case 0x6b:
		case 0x6f:
		case 0xeb:
		case 0xef:
			return WALL_F;
			break;
			
		case 0x78:
		case 0x79:
		case 0x7c:
		case 0x7d:
			return WALL_AH;
			break;
			
		case 0x7a:
			return WALL_ACD;
			break;
			
		case 0x7b:
			return WALL_AC;
			break;
			
		case 0x7e:
			return WALL_AD;
			break;
			
		case 0x7f:
			return WALL_A;
			break;
			
		case 0xd0:
		case 0xd1:
		case 0xd4:
		case 0xd5:
		case 0xf0:
		case 0xf1:
		case 0xf4:
		case 0xf5:
			return WALL_GH;
			break;
			
		case 0xd2:
		case 0xd3:
		case 0xf2:
		case 0xf3:
			return WALL_CG;
			break;
			
		case 0xd6:
		case 0xd7:
		case 0xf6:
		case 0xf7:
			return WALL_G;
			break;
			
		case 0xd8:
		case 0xd9:
		case 0xdc:
		case 0xdd:
			return WALL_BH;
			break;
			
		case 0xda:
			return WALL_BCD;
			break;
			
		case 0xdb:
			return WALL_BC;
			break;
			
		case 0xde:
			return WALL_BD;
			break;
			
		case 0xdf:
			return WALL_B;
			break;
			
		case 0xf8:
		case 0xf9:
		case 0xfc:
		case 0xfd:
			return WALL_H;
			break;
			
		case 0xfa:
			return WALL_CD;
			break;
			
		case 0xfb:
			return WALL_C;
			break;
			
		case 0xfe:
			return WALL_D;
			break;
		
		case 0xff:
		default:
			return WALL_WHOLE;
	}
}

void set_correct_city_wall( int i, int j, plv Level )
{
	short c, locvec;
	int x, y;
	
	locvec = 0;
	for ( y = j-1; y < j+2; y++ )
		for ( x = i-1; x < i+2; x++ )
		{
			if ( x < 0 || y < 0 || x >= WIDTH || y >= LENGTH )
			{
				locvec <<= 1;
				locvec |= 0x01;
			}
			else if ( !( x == i && y == j ) )
			{
				locvec <<= 1;
				c = Level->site[ x ][ y ].locchar;
				/*
				if (( c >= WALL_FIRST && c <= WALL_LAST ) ||
					( c >= WALL_DIAG_FIRST && c <= WALL_DIAG_LAST ) ||
					c == MY_OPEN_DOOR || c == MY_CLOSED_DOOR ||
					loc_statusp( x, y, SECRET ))
				*/
				if (( c >= WALL_FIRST && c <= WALL_LAST ) ||
					( c >= WALL_DIAG_FIRST && c <= WALL_DIAG_LAST ) ||
					c == MY_OPEN_DOOR || c == MY_CLOSED_DOOR ||
					loc_statusp( x, y, SECRET ) || !loc_statusp( x, y, SEEN ))
					/* ( !loc_statusp( x, y, SEEN ) && ( c == GROUND ))) */
				{
					locvec |= 0x01;
				}
			}
		}
		
	Level->site[ i ][ j ].locchar = assign_wall_char( locvec );
	/*
	if ( loc_statusp( i, j, SEEN ))
	{
		Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
		return;
	}
	
	locvec = 0;
	for ( y = j-1; y < j+2; y++ )
		for ( x = i-1; x < i+2; x++ )
		{
			if ( x < 0 || y < 0 || x >= WIDTH || y >= LENGTH )
			{
				locvec <<= 1;
				locvec |= 0x01;
			}
			else if ( !( x == i && y == j ) )
			{
				locvec <<= 1;
				c = Level->site[ x ][ y ].locchar;
				if (( c >= WALL_FIRST && c <= WALL_LAST ) ||
					( c >= WALL_DIAG_FIRST && c <= WALL_DIAG_LAST ) ||
					c == MY_OPEN_DOOR || c == MY_CLOSED_DOOR ||
					loc_statusp( x, y, SECRET ) ||
					( !loc_statusp( x, y, SEEN ) && !( c >= HEDGE_FIRST && c <= HEDGE_LAST )))
				{
					locvec |= 0x01;
				}
			}
		}
		
	Level->site[ i ][ j ].showchar = assign_wall_char( locvec );
	*/
}

void fixup_walls( int x, int y, plv Level )
{
	unsigned short saveloc;
	int i, j;
	
	for ( i = max( 0, x-1 ); i < min( WIDTH, x+2 ); i++ )
		for ( j = max( 0, y-1 ); j < min( LENGTH, y+2 ); j++ )
		{
			if ( Level->site[ i ][ j ].locchar == WALL ||
				 ( Level->site[ i ][ j ].locchar >= WALL_FIRST &&
				   Level->site[ i ][ j ].locchar <= WALL_LAST ))
			{
				set_correct_city_wall( i, j, Level );
				plotspot( i, j, FALSE );
			}
			else if ( loc_statusp( i, j, SEEN ))
			{
				if ( loc_statusp( i, j, SECRET ))
				{
					saveloc = Level->site[ i ][ j ].locchar;
					set_correct_city_wall( i, j, Level );
					Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
					Level->site[ i ][ j ].locchar = saveloc;
				}
				else
					Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
					
				plotspot( i, j, FALSE );
			}
		}
		
	refresh();
}

void fixup_hedges( int x, int y, plv Level )
{
	int i, j;
	
	for ( i = max( 0, x-1 ); i < min( WIDTH, x+2 ); i++ )
		for ( j = max( 0, y-1 ); j < min( LENGTH, y+2 ); j++ )
		{
			if ( Level->site[ i ][ j ].locchar == HEDGE ||
				 ( Level->site[ i ][ j ].locchar >= HEDGE_FIRST &&
				   Level->site[ i ][ j ].locchar <= HEDGE_LAST ))
			{
				set_correct_hedge( i, j, Level );
				plotspot( i, j, FALSE );
			}
			else if ( loc_statusp( i, j, SEEN ))
			{
				Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
				plotspot( i, j, FALSE );
			}
		}
	refresh();
}

void fixup_pools( int x, int y, plv Level )
{
	int i, j;
	
	for ( i = max( 0, x-1 ); i < min( WIDTH, x+2 ); i++ )
		for ( j = max( 0, y-1 ); j < min( LENGTH, y+2 ); j++ )
		{
			if ( Level->site[ i ][ j ].locchar == WATER ||
				 ( Level->site[ i ][ j ].locchar >= POOL_FIRST &&
				   Level->site[ i ][ j ].locchar <= POOL_LAST ))
			{
				set_correct_pool( i, j, Level );
				plotspot( i, j, FALSE );
			}
			else if ( loc_statusp( i, j, SEEN ))
			{
				Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
				plotspot( i, j, FALSE );
			}
		}
	refresh();
}

void set_correct_tile( int i, int j, plv Level )
{
	short saveloc;
	
	if ( Level->site[ i ][ j ].locchar == WALL ||
		 ( Level->site[ i ][ j ].locchar >= WALL_FIRST &&
		   Level->site[ i ][ j ].locchar <= WALL_LAST ))
	{
		set_correct_city_wall( i, j, Level );
	}
	else if ( Level->site[ i ][ j ].locchar == WATER ||
			  ( Level->site[ i ][ j ].locchar >= POOL_FIRST &&
				Level->site[ i ][ j ].locchar <= POOL_LAST ))
	{
		set_correct_pool( i, j, Level );
	}
	else if ( Level->site[ i ][ j ].locchar == HEDGE ||
			  ( Level->site[ i ][ j ].locchar >= HEDGE_FIRST &&
				Level->site[ i ][ j ].locchar <= HEDGE_LAST ))
	{
		set_correct_hedge( i, j, Level );
	}
	else if ( loc_statusp( i, j, SEEN ))
	{
		if ( loc_statusp( i, j, SECRET ))
		{
			saveloc = Level->site[ i ][ j ].locchar;
			set_correct_city_wall( i, j, Level );
			Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
			Level->site[ i ][ j ].locchar = saveloc;
		}
		else
			Level->site[ i ][ j ].showchar = Level->site[ i ][ j ].locchar;
	}
}
